@@ -2,11 +2,11 @@
 
                           Firewall Builder
 
-                 Copyright (C) 2000 Vadim Kurland
+                 Copyright (C) 2000 NetCitadel, LLC
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: ASDruidP05Dialog.cc,v 1.5 2001/09/22 01:39:29 vkurland Exp $
+  $Id: ASDruidP05Dialog.cc,v 1.9 2002/09/04 22:33:43 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -25,6 +25,7 @@
 */
 
 
+#include "fwbuilder/libfwbuilder-config.h"
 
 #include "ASDruidP05Dialog.hh"
 
@@ -34,9 +35,9 @@ using namespace libfwbuilder;
 
 ASDruidP05Dialog::ASDruidP05Dialog()
 {
-    P05_lbl1->set_line_wrap(true);
-    P05_lbl2->set_line_wrap(true);
-    P05_lbl3->set_line_wrap(true);
+//    P05_lbl1->set_line_wrap(true);
+//    P05_lbl2->set_line_wrap(true);
+//    P05_lbl3->set_line_wrap(true);
 }
 
 
