@@ -8,14 +8,14 @@
 #include "config.h"
 #include "ASDruidP47Dialog.hh"
 #include "helpers.hh"
-#include "fwbuilder/FWObjectDatabase.hh"
+#include "FWObjectDatabaseGUI.hh"
 
 using namespace libfwbuilder;
 
 ASDruidP47Dialog::ASDruidP47Dialog()
 {
     FWObject *frag;
-    frag=_findObjectByName(FWObjectDatabase::db->std.ServicesId,"ip_fragments");
+    frag=_findObjectByName(FWObjectDatabaseGUI::getServicesId(),"ip_fragments");
     if (frag==NULL) short_fragments->set_sensitive(false);
 
     setButtonLblAlignment(anti_spoof_in   , 0.0, 0.5);
