@@ -2,11 +2,11 @@
 
                           Firewall Builder
 
-                 Copyright (C) 2000 Vadim Kurland
+                 Copyright (C) 2000 NetCitadel, LLC
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyListElement.cc,v 1.49 2001/12/27 06:48:47 vkurland Exp $
+  $Id: PolicyListElement.cc,v 1.83 2003/03/22 09:03:43 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -26,33 +26,42 @@
 
 
 
-#include <iostream.h>
+#include <iostream>
 
 #include "config.h"
 
-#include "glademm_support.hh"
-#include "helpers.hh"
 #include "fwbuilder/FWObjectDatabase.hh"
-#include "BuiltinDialog.hh"
-#include "gen_popup_menu.hh"
 #include "fwbuilder/Host.hh"
 #include "fwbuilder/Firewall.hh"
-#include "Resources.hh"
+#include "fwbuilder/FWOptions.hh"
+#include "fwbuilder/FWObjectReference.hh"
+#include "fwbuilder/FWServiceReference.hh"
+#include "fwbuilder/FWIntervalReference.hh"
+#include "fwbuilder/Resources.hh"
+
+#include "helpers.hh"
+#include "platforms.hh"
+#include "BuiltinDialog.hh"
+#include "popupMenu.hh"
+
 #include "Preferences.hh"
 #include "CommentDialog.hh"
 #include "RuleOptionsDialog.hh"
 
 #include "FWObjectDatabaseGUI.hh"
 
-#include "fwbuilder/FWObjectReference.hh"
-#include "fwbuilder/FWServiceReference.hh"
-#include "fwbuilder/FWIntervalReference.hh"
-
 #include "PolicyListElement.hh"
 #include "PolicyListItem.hh"
 #include "PolicyList.hh"
 
-#include "fwbuilder/FWOptions.hh"
+#include "main_window.hh"
+
+#include "glademm_support.hh"
+
+#include <iostream>
+
+#include <assert.h>
+
 
 using namespace libfwbuilder;
 
@@ -67,29 +76,38 @@ PolicyListElement::PolicyListElement(gint r,gint c) : Gtk::Frame()
     row=r;
     col=c;
     width=height=0;
+    neg=false;
 
     set_flags( GTK_CAN_FOCUS );
 
     set_name("PolicyListElement");
 
-    vbox=manage(new Gtk::VBox());
+    vbox=new Gtk::VBox();
     add(*vbox);
     vbox->show();
 
+    popup_menu=NULL;
+
+#ifdef __MINGW32__
+/* see comment in PolicyListItem.cc about these flags */
+    drag_dest_set ( GtkDestDefaults(GTK_DEST_DEFAULT_DROP | GTK_DEST_DEFAULT_MOTION),
+		    target_table, n_targets , 
+		    static_cast<GdkDragAction>(GDK_ACTION_COPY) );
+#else
     drag_dest_set ( GTK_DEST_DEFAULT_ALL,
 		    target_table, n_targets , 
 		    static_cast<GdkDragAction>(GDK_ACTION_COPY) );
-    
+#endif
 
     rel=NULL;
 }
 
 PolicyListElement::PolicyListElement(gint r,gint c, RuleElement *re) : Gtk::Frame()
 {
-    PolicyListItem    *itm;
     FWObject          *o, *oref;
 
     rel=re;
+    neg=rel->getNeg();
 
     row=r;
     col=c;
@@ -99,22 +117,31 @@ PolicyListElement::PolicyListElement(gint r,gint c, RuleElement *re) : Gtk::Fram
 
     set_name("PolicyListElement");
 
-    vbox=manage(new Gtk::VBox());
+    vbox=new Gtk::VBox();
     add(*vbox);
     vbox->show();
 
+    popup_menu=NULL;
+
+#ifdef __MINGW32__
+/* see comment in PolicyListItem.cc about these flags */
+    drag_dest_set ( GtkDestDefaults(GTK_DEST_DEFAULT_DROP | GTK_DEST_DEFAULT_MOTION),
+		    target_table, n_targets , 
+		    static_cast<GdkDragAction>(GDK_ACTION_COPY) );
+#else
     drag_dest_set ( GTK_DEST_DEFAULT_ALL,
 		    target_table, n_targets , 
 		    static_cast<GdkDragAction>(GDK_ACTION_COPY) );
-    
+#endif
+
     change_any_to_orig=false;
     if (RuleElementTSrc::isA(re) ||
 	RuleElementTDst::isA(re) ||
 	RuleElementTSrv::isA(re) )  change_any_to_orig=true;
     
 
-    vector<FWObject*>::iterator m;
-    for (m = rel->begin(); m!= rel->end(); ++m) {
+    for(list<FWObject*>::iterator  m = rel->begin(); m!= rel->end(); ++m) 
+    {
 	o=(*m);
 	oref=NULL;
 	if (FWObjectReference::isA(o) || FWServiceReference::isA(o) ||
@@ -122,11 +149,9 @@ PolicyListElement::PolicyListElement(gint r,gint c, RuleElement *re) : Gtk::Fram
 	    oref=(FWReference::cast(o))->getPointer();
 
 	if (oref!=NULL) {
-	    itm= manage(new PolicyListObjectItem(oref, 
-						 rel->getNeg(), 
-						 change_any_to_orig) );
-	    vbox->pack_start( *itm , true , true , 0 );
-	    itm->show();
+	    addPolicyListItem( new PolicyListObjectItem(oref, 
+                                                        rel->getNeg(), 
+                                                        change_any_to_orig) );
 	}
 
     }
@@ -135,6 +160,53 @@ PolicyListElement::PolicyListElement(gint r,gint c, RuleElement *re) : Gtk::Fram
     show_all();
 }
 
+PolicyListElement::~PolicyListElement()
+{
+    while ( get_first_child()!=NULL )
+        removePolicyListItem();
+
+    remove();
+    delete vbox;
+
+    if (popup_menu!=NULL) delete popup_menu;
+}
+
+RuleSetList* PolicyListElement::getParentRuleSetList()
+{
+    Gtk::Widget *w=this;
+    while (w!=NULL && dynamic_cast<RuleSetList*>(w)==NULL) 
+        w=w->get_parent();
+
+    return (RuleSetList*)w;
+}
+
+void PolicyListElement::addPolicyListItem(PolicyListItem *pi)
+{
+//    manage(pi);
+    vbox->pack_start( *pi , true , true , 0 );
+    pi->show();
+//    Gdk_Window gdkw=pi->get_window();
+//    cerr << "added pi " << pi << " Gdk_Window=" << gdkw << endl;
+}
+
+void PolicyListElement::removePolicyListItem(PolicyListItem *pi)
+{
+    if (pi==NULL)
+    {
+        Gtk::Widget *w=get_first_child();
+        if (w) 
+        { 
+            w->hide();
+            vbox->remove(*w); 
+            delete w;
+        }
+    } else
+    {
+        pi->hide();
+        vbox->remove(*pi);
+        delete pi;
+    }
+}
 
 Gtk::Widget* PolicyListElement::get_first_child()
 {
@@ -183,28 +255,23 @@ void PolicyListElement::remove_item_from_policy(PolicyListItem* pi)
     FWObject *obj = pi->getObject();
     assert(obj!=NULL);
 
-    vbox->remove(*pi);   // do not need to delete because it is managed
+    removePolicyListItem(pi);
+//    vbox->remove(*pi);   // do not need to delete because it is managed
     
     rel->removeRef(obj);  // adds "any" if obj was the last one
 
     if (rel->isAny()) {
+	rel->setNeg(false);
+	neg=false;
 
 	obj=obj->getRoot()->getById( rel->getAnyElementId() , true );
 
-	PolicyListItem* itm= 
-	    manage(new PolicyListObjectItem( obj, 
-					     rel->getNeg(), 
-					     change_any_to_orig) );
-	vbox->pack_start( *itm , true , true , 0 );
-	itm->show();
-
+	addPolicyListItem( new PolicyListObjectItem( obj, false,
+                                                     change_any_to_orig) );
 	select();
-
-	if (rel->getNeg())  toggleNeg();
-    }	
+    }
 
     selected_item=get_first_child();
-
     set_data_changed_flag(true);
 }
 
@@ -215,13 +282,24 @@ bool PolicyListElement::add_item_to_policy(FWObject *obj)
 
     if (rel->getAnyElementId()==obj->getId()) return(false);
 
-/*
- * we do not want duplicates
- * calling getById with third parameter to follow references
- */
-    if (rel->getById(obj->getId(),true,true)!=NULL) return(false);
+    string obj_id=obj->getId();
+    bool present=false;
+    list<FWObject*>::iterator j;
+    for(j=rel->begin(); j!=rel->end(); ++j)     
+    {
+        FWObject *o=*j;
+        string oid=o->getId();
+        if(obj_id==oid) { present=true; break; }
+
+        FWReference *ref;
+        if( (ref=FWReference::cast(o))!=NULL &&
+            obj_id==ref->getPointerId()) { present=true; break; }
+    }
+    if (present) return false;
+
     
-    if (rel->isAny()) {
+    if (rel->isAny()) 
+    {
 	// element contains "Any". Remove corresponding PolicyListItem
 	// from PolicyListElement before adding new item 
 	//
@@ -238,8 +316,7 @@ bool PolicyListElement::add_item_to_policy(FWObject *obj)
 	    vbox->remove( *cc );
 	}
 */
-	Gtk::Widget *w=get_first_child();
-	if (w) vbox->remove(*w);
+        removePolicyListItem();
 
 	selected_item=NULL;
 
@@ -251,11 +328,7 @@ bool PolicyListElement::add_item_to_policy(FWObject *obj)
 
     deselect();
 
-    PolicyListItem* itm= 
-	manage(new PolicyListObjectItem( obj, 
-					 rel->getNeg(),change_any_to_orig) );
-    vbox->pack_start( *itm , true , true , 0 );
-    itm->show();
+    addPolicyListItem( new PolicyListObjectItem( obj, neg, change_any_to_orig) );
 	
     selected_item=get_last_child();
 	
@@ -267,13 +340,15 @@ bool PolicyListElement::add_item_to_policy(FWObject *obj)
 }
 
 
-void PolicyListElement::toggleNeg()
+
+void PolicyListElement::setNeg(bool val)
 {
-    rel->toggleNeg();
+    neg=val;
+    if (rel!=NULL) rel->setNeg(val);
 
     Gtk::Box_Helpers::BoxList::iterator bl_i;
     for (bl_i=vbox->children().begin(); bl_i!=vbox->children().end(); bl_i++) {
-	((PolicyListItem*)(*bl_i)->get_widget())->setNeg( rel->getNeg() );
+	((PolicyListItem*)(*bl_i)->get_widget())->setNeg( val );
     }
 
     set_data_changed_flag(true);
@@ -301,7 +376,7 @@ void PolicyListElement::deactivate()
 void PolicyListElement::select(bool select_row)
 {
     if (select_row) {
-	PolicyList* pp=(PolicyList*)get_parent();
+	RuleSetList* pp=getParentRuleSetList();
     
 	pp->deselect_current();
 	pp->activate_row(row);
@@ -417,14 +492,14 @@ void PolicyListElement::drag_data_received_impl(GdkDragContext   *context,
 
 gint PolicyListElement::key_press_event_impl(GdkEventKey* ev)
 {
-    PolicyList* pp=(PolicyList*)get_parent();
+    RuleSetList* pp=getParentRuleSetList();
 
     if ( selected_item==NULL ) goto another_element;
 
     switch (ev->keyval) {
 
     case GDK_space:
-	popup_menu( (PolicyListItem*)selected_item );
+	show_popup_menu( (PolicyListItem*)selected_item );
 	break;
 
     case GDK_Up:
@@ -461,22 +536,6 @@ gint PolicyListElement::key_press_event_impl(GdkEventKey* ev)
 	else goto another_element;
 	break;
     }
-    case GDK_F5:
-    {
-	cerr << "------- PolicyListElement dump:\n";
-	cerr << "row=" << get_row() << "  col=" << get_col() << endl;
-	Gtk::Box_Helpers::BoxList::iterator bl_i;
-	Gtk::Box_Helpers::Child    *ch;
-	Gtk::Widget                *cc;
-	for (bl_i=vbox->children().begin(); bl_i!=vbox->children().end(); ++bl_i) {
-	    ch= *bl_i;
-	    cc=ch->get_widget();
-	    cerr << " child widget: " << cc << endl;
-	}
-	rel->dump(true,true);
-
-	break;
-    }
 
     default:
 	
@@ -494,79 +553,158 @@ gint PolicyListElement::key_press_event_impl(GdkEventKey* ev)
     return(true);
 }
 
-void PolicyListElement::popup_menu(PolicyListItem *itm)
+void PolicyListElement::create_popup_menu()
 {
-    FWObject               *obj;
-    gen_popup_menu         *gpm;
-    PolicyList             *pl=(PolicyList*)get_user_data();
+    if (popup_menu==NULL)
+    {
+        const char *menu_items0[] = 
+            { _("Edit"),
+              _("Edit in another window"),
+              "",
+              _("Copy"),
+              _("Cut"),
+              _("Paste"),
+              "",
+              _("Negate"),
+              NULL } ;
 
-    g_assert(rel!=NULL);
+        popup_menu=new popupMenu(menu_items0);
+        popup_menu->menu_select.connect(SigC::slot(this,&PolicyListElement::popup_menu_callback));
+    }
+}
 
+void PolicyListElement::show_popup_menu(PolicyListItem *itm)
+{
+    popup_menu_pi=itm;
 
+    create_popup_menu();
 
-    obj=itm->getObject();
+    RuleSetList        *pl=(RuleSetList*)get_user_data();
+    assert(rel!=NULL);
 
-    const char *menu_items0[] = 
-    { "Edit","","Copy","Cut","Paste","","Negate",NULL } ;
-    gpm=new gen_popup_menu( menu_items0 );
+    FWObject *f=rel;
+    while (f!=NULL && !Firewall::isA(f)) f=f->getParent();
+    assert(f!=NULL);
 
-    if ( rel->isAny() ) {
-	Gtk::Widget *menu_itm;
-	menu_itm=find_widget("Edit",gpm);   
-	if (menu_itm) menu_itm->set_sensitive(false);
-	menu_itm=find_widget("Negate",gpm); 
-	if (menu_itm) menu_itm->set_sensitive(false);
-	menu_itm=find_widget("Copy",gpm);   
-	if (menu_itm) menu_itm->set_sensitive(false);
-	menu_itm=find_widget("Cut",gpm);    
-	if (menu_itm) menu_itm->set_sensitive(false);
+// f is a pointer at firewall object
+
+    string cap_name;
+    if (dynamic_cast<InterfacePolicyList*>(pl)!=NULL) cap_name="negation_in_interface_policy";
+    if (dynamic_cast<PolicyList*>(pl)!=NULL)          cap_name="negation_in_policy";
+    if (dynamic_cast<NATList*>(pl)!=NULL)             cap_name="negation_in_nat";
+
+    bool supports_neg=false;
+    try  {
+        supports_neg=Resources::getTargetCapabilityBool(f->getStr("platform"),
+                                                        cap_name);
+    } catch (FWException &ex)
+    {
+        MessageDialog::Error(ex.toString());
     }
 
-    gpm->popup(0,0);
-    gint menu_choice=gpm->run();
-    delete gpm;
+    popup_menu->set_sensitive(0,!rel->isAny());
+    popup_menu->set_sensitive(1,!rel->isAny());
+    popup_menu->set_sensitive(3,!rel->isAny());
+    popup_menu->set_sensitive(4,!rel->isAny());
 
-    deselect();
+    popup_menu->set_sensitive(7,supports_neg && !rel->isAny());
+
+    popup_menu->popup(0,0);
+}
+
+void PolicyListElement::popup_menu_callback(int menu_choice)
+{
+    assert(rel!=NULL);
+
+    FWObject *obj=popup_menu_pi->getObject();
+
+    /*
+     * the problem here is that RuleElement object (pointer 'rel') may
+     * change inside the call to safe_to_close_right_pane(). This
+     * happens because it presents user with choice "discard
+     * changes". This operation copies the top level Policy object to
+     * restore data and thus recreates all its children, including this
+     * Rule Element. We need to make sure we do not touch it after the call
+     * to safe_to_close_right_pane()   
+     *
+     * TODO:
+     *
+     * FWObject::operaror= should really be rewritten so that it won't
+     * destroy all its children before copying data, but instead copy
+     * data for those that do exist and create new ones when
+     * needed. This is going to be lot more complex algorithm though,
+     * and most likely slow, too.
+     *
+     * --vk 02/09/02
+     */
+    bool is_any=rel->isAny();
 
     switch (menu_choice) {
     case 0:    // edit
 /*
- *  pass object via clipboard to avoid passing reference to the stack
- *
  *  do not edit "Any"
  */
-	if ( obj!=NULL &&  ! rel->isAny() ) {
+        if (obj!=NULL && !is_any) 
+        {
+            main_window *mw=main_window::getMainWindowForWidget(this);
+            if (mw->safe_to_close_right_pane() )
+		mw->schedule_open_object(obj->getId(), obj->getLibrary() );
+        }
+	break;
+    case 1:    // edit in another window
+/*
+ *  do not edit "Any"
+ */
+        if (obj!=NULL && !is_any) {
+            main_window *mw=main_window::getAnotherMainWindowForWidget(this);
+            if (mw->safe_to_close_right_pane() )
+		mw->schedule_open_object(obj->getId(), obj->getLibrary() );
+        }
+	break;
+    case 3:    // copy
+	if ( obj!=NULL && ! is_any )
 	    FWObjectClipboard::obj_clipboard->putObject( obj );
-// open_item is a signal
-	    pl->open_item( FWObjectClipboard::obj_clipboard->getObject() );
-	}
-	return;
-    case 2:    // copy
-	if ( obj!=NULL && ! rel->isAny() ) {
+
+	break;
+    case 4:    // cut
+	if ( obj!=NULL && ! is_any ) {
 	    FWObjectClipboard::obj_clipboard->putObject( obj );
+	    remove_item_from_policy(popup_menu_pi);
 	}
-	return;
-    case 3:    // cut
-	if ( obj!=NULL && ! rel->isAny() ) { 
-	    FWObjectClipboard::obj_clipboard->putObject( obj );
-	    remove_item_from_policy(itm);
-	}
-	return;
-    case 4:    // paste
+	break;
+    case 5:    // paste
     {
+        /*
+         * clipboard holds a copy of the original object. Find an
+         * original object, check if it still belongs to a tree and
+         * add reference to it to the rule element. If user did "cut"
+         * and object in the clipboard is not attached to the tree
+         * anymore, then we can't paste it into the rule.
+         */
 	obj=FWObjectClipboard::obj_clipboard->getObject();
 	if ( obj!=NULL ) {
-	    add_item_to_policy(obj);
+	    FWObject *oo=FWObjectDatabaseGUI::db->getById( obj->getId() , true);
+            if (oo!=NULL) add_item_to_policy(oo);
 	    set_data_changed_flag(true);
 	}
     }
-    return;
+    break;
 
-    case 6:    // negate
-	toggleNeg();   
-	return;
+    case 7:    // negate
+	if ( ! is_any ) setNeg( ! getNeg() );
+	break;
     }
-    select();
+}
+
+void PolicyListElement::init() 
+{
+    build();
+    selected_item=get_first_child();
+//    drag_dest_unset();
+}
+
+void PolicyListElement::build() 
+{
 }
 
 /*******************************************************************/
@@ -574,23 +712,63 @@ void PolicyListElement::popup_menu(PolicyListItem *itm)
 static int last_drag_motion_x=-1;
 
 
-PolicyListRuleNum::PolicyListRuleNum(gint r,gint c,gint rule_n,bool neg) :
+PolicyListRuleNum::PolicyListRuleNum(gint r,gint c,gint _num,bool _neg) :
     PolicyListElement(r,c)
 {
-    PolicyListRuleNumItem    *itm;
+    rule_n=_num;
+    neg   =_neg;
+    set_name("PolicyListRuleNum");
+}
+
+void PolicyListRuleNum::create_popup_menu()
+{
+    if (popup_menu==NULL)
+    {
+        const char *menu_items[] = 
+
+            { _("Insert new rule above"),
+              _("Add new rule below"),
+              _("Remove this rule"),
+              _("Move this rule up"),
+              _("Move this rule down"),
+              "",
+              _("Copy"),
+              _("Cut"),
+              _("Paste above"),
+              _("Paste below"),
+              "",
+              _("Enable"),
+              _("Disable"),
+              NULL } ;
+
+        popup_menu=new popupMenu(menu_items);
+        popup_menu->menu_select.connect(SigC::slot(this,&PolicyListRuleNum::popup_menu_callback));
+    }
+}
+
+void PolicyListRuleNum::build() 
+{
+//    deselect();
     char  str[64];
 
     sprintf(str,"%02d",rule_n);
-    itm= manage(new PolicyListRuleNumItem( str , neg ));
-    vbox->pack_start( *itm , true , true , 0 );
-    itm->show();
-
-    selected_item=get_first_child();
-    set_name("PolicyListRuleNum");
+    addPolicyListItem(new PolicyListRuleNumItem( str , neg ));
 
     last_drag_motion_x=-1;
 }
 
+void PolicyListRuleNum::setNum(int rule_n)
+{
+    char  str[64];
+    sprintf(str,"%02d",rule_n);
+
+    Gtk::Widget *w=get_first_child();
+    if (dynamic_cast<PolicyListRuleNumItem*>(w))
+	dynamic_cast<PolicyListRuleNumItem*>(w) ->setNum(str);
+
+}
+
+
 gboolean PolicyListRuleNum::drag_motion_impl(GdkDragContext* context,
 					     gint x,gint y,guint time)
 {
@@ -640,48 +818,32 @@ void PolicyListRuleNum::drag_data_received_impl(GdkDragContext   *context,
 
 /* rule number src_rule moves right above rule dst_rule */
 	    PolicyList         *pl=(PolicyList*)get_user_data();
-	    RuleSet            *pol=pl->getRuleSet();
-	    if ( pol->moveRule(src_rule,dst_rule) ) {
-		pl->schedule_rebuild((src_rule<dst_rule)?dst_rule-1:dst_rule);
-		set_data_changed_flag(true);
-	    }
+	    pl->moveRuleTo(src_rule,dst_rule);
 	    return;
 	}
     }
 }
 
       
+void PolicyListRuleNum::show_popup_menu(PolicyListItem *itm)
+{
+    PolicyList             *pl=(PolicyList*)get_user_data();
+    gint rule = PolicyList::get_rule_by_row( get_row() );
 
-void PolicyListRuleNum::popup_menu(PolicyListItem *itm)
+    create_popup_menu();
+
+    popup_menu->set_sensitive(11, pl->isRuleDisabled( rule ));
+    popup_menu->set_sensitive(12,!pl->isRuleDisabled( rule ));
+
+    popup_menu->popup(0,0);
+}
+
+void PolicyListRuleNum::popup_menu_callback(int menu_choice)
 {
-    gen_popup_menu         *gpm;
     PolicyList             *pl=(PolicyList*)get_user_data();
     RuleSet                *rs=pl->getRuleSet();
     gint rule = PolicyList::get_rule_by_row( get_row() );
 
-    const char *menu_items0[] = 
-    { "Insert new rule above",
-      "Add new rule below",
-      "Remove this rule",
-      "Move this rule up",
-      "Move this rule down",
-      "",
-      "Copy",
-      "Cut",
-      "Paste above",
-      "Paste below",
-      "",
-      NULL,
-      NULL } ;
-
-    if (pl->isRuleDisabled( rule ))  menu_items0[11]="Enable";
-    else                             menu_items0[11]="Disable";
-
-    gpm=new gen_popup_menu( menu_items0 );
-    gpm->popup(0,0);
-    gint menu_choice=gpm->run();
-    delete gpm;
-
     deselect();
 
     switch (menu_choice) {
@@ -695,8 +857,8 @@ void PolicyListRuleNum::popup_menu(PolicyListItem *itm)
 	break;
     case 2:
 	pl->delRule( rule );
-	set_data_changed_flag(true);
-	break;
+	pl->set_data_changed_flag(true);
+	return;    // RETURN immediately since this rule element belongs to the rule which we delete
     case 3:
 	pl->moveRuleUp( rule );
 	set_data_changed_flag(true);
@@ -709,17 +871,17 @@ void PolicyListRuleNum::popup_menu(PolicyListItem *itm)
     case 6:  // copy
     {
 	Rule *r=rs->getRuleByNum(rule);
-	FWObjectClipboard::obj_clipboard->putObject( r );
+	FWObjectClipboard::obj_clipboard->putObject( r );  // creates a copy
     }
     break;
 
     case 7:  // cut
     {
 	Rule *r=rs->getRuleByNum(rule);
-
-	FWObjectClipboard::obj_clipboard->putObject( r );
+	FWObjectClipboard::obj_clipboard->putObject( r );  // creates a copy
 	pl->delRule( rule );
-	set_data_changed_flag(true);
+	pl->set_data_changed_flag(true);
+	return;    // RETURN immediately since this rule element belongs to the rule which we delete
     }
     break;
 
@@ -746,9 +908,15 @@ void PolicyListRuleNum::popup_menu(PolicyListItem *itm)
     break;
 
     case 11:
-	if (pl->isRuleDisabled( rule )) pl->enableRule( rule );
-	else                            pl->disableRule( rule );
+	pl->enableRule( rule );
 	set_data_changed_flag(true);
+        break;
+
+    case 12:
+        pl->disableRule( rule );
+	set_data_changed_flag(true);
+        break;
+
     }
     select();
 }
@@ -760,56 +928,70 @@ PolicyListRuleAction::PolicyListRuleAction(gint r,gint c, Rule *ru)
   :    PolicyListElement(r,c)
 {
     rule=ru;
-    rebuild();
-    selected_item=get_first_child();
+
 }
-      
-void PolicyListRuleAction::popup_menu(PolicyListItem *itm)
+
+void PolicyListRuleAction::create_popup_menu()
 {
-    gen_popup_menu         *gpm;
+    FWObject     *f=((RuleSetList*)get_user_data())->getRuleSet();
+    while (f!=NULL && !Firewall::isA(f)) f=f->getParent();
+    assert(f!=NULL);
 
-    const char *menu_choices[] = 
-    { "Accept",
-      "Deny",
-      "Reject",
-      NULL } ;
-    gpm=new gen_popup_menu( menu_choices );
-    gpm->popup(0,0);
-    gint res=gpm->run();
-    delete gpm;
+    bool supports_acct=false;
+    try  {
+        supports_acct=Resources::getTargetCapabilityBool(f->getStr("platform"),
+                                                         "supports_accounting");
+    } catch (FWException &ex)    {    }
 
-    if (res>=0) {
-	string s=menu_choices[res];
-	rule->setStr("action",s);
-	rebuild();
+
+    if (popup_menu==NULL)
+    {
+        const char *menu_items[] = 
+            { _("Accept"),
+              _("Deny"),
+              _("Reject"),
+              _("Accounting"),
+              NULL } ;
+
+        if (!supports_acct) menu_items[3]=NULL;
+
+        popup_menu=new popupMenu(menu_items);
+        popup_menu->menu_select.connect(SigC::slot(this,&PolicyListRuleAction::popup_menu_callback));
+    }
+}
+ 
+void PolicyListRuleAction::show_popup_menu(PolicyListItem *itm)
+{
+    create_popup_menu();
+    popup_menu->popup(0,0);
+}
+
+void PolicyListRuleAction::popup_menu_callback(int menu_choice)
+{
+    if (menu_choice>=0) {
+        switch (menu_choice)
+        {
+        case 0: PolicyRule::cast(rule)->setAction(PolicyRule::Accept);     break;
+        case 1: PolicyRule::cast(rule)->setAction(PolicyRule::Deny);       break;
+        case 2: PolicyRule::cast(rule)->setAction(PolicyRule::Reject);     break;
+        case 3: PolicyRule::cast(rule)->setAction(PolicyRule::Accounting); break;
+        }
+	build();
 
 	set_data_changed_flag(true);
     }
-
 }
 
-void PolicyListRuleAction::rebuild() 
+void PolicyListRuleAction::build() 
 {
-    bool o=false;
+    removePolicyListItem();
 
-    deselect();
-
-    PolicyListItem *itm;
-
-    Gtk::Widget *w=get_first_child();
-    if (w) { o=true; vbox->remove(*w); }
-
-    string action=rule->getStr("action");
-    if (action=="") action="Deny";
+    string action= PolicyRule::cast(rule)->getActionAsString();
 
     string icn=	Resources::global_res->getIconPath(action);
     assert(icn!="");
 
-    itm= manage(new PolicyListItem( icn,action ));
-    vbox->pack_start( *itm , true , true , 0 );
-    itm->show();
-
-    if (o) itm->set_state(GTK_STATE_SELECTED);
+    addPolicyListItem(new PolicyListItem( icn,"",action ));
 }
 
 /*******************************************************************/
@@ -819,61 +1001,66 @@ PolicyListRuleLog::PolicyListRuleLog(gint r,gint c, Rule *ru) :
     PolicyListElement(r,c)
 {
     rule=ru;
-    rebuild();
-    selected_item=get_first_child();
+
 }
 
-void PolicyListRuleLog::rebuild() 
+void PolicyListRuleLog::create_popup_menu()
 {
-    bool o=false;
+    if (popup_menu==NULL)
+    {
+        const char *menu_items[] = 
+            { _("Log"),
+              _("Do not log"),
+              NULL } ;
 
-    deselect();
+        popup_menu=new popupMenu(menu_items);
+        popup_menu->menu_select.connect(SigC::slot(this,&PolicyListRuleLog::popup_menu_callback));
+    }
+}
 
+
+void PolicyListRuleLog::build() 
+{
     PolicyListItem *itm;
 
-    Gtk::Widget *w=get_first_child();
-    if (w) { o=true; vbox->remove(*w); }
+    removePolicyListItem();
 
-    itm= manage(new PolicyListItem( "","" ));
+    itm= new PolicyListItem( "","","" );
 
-    bool    log=rule->getBool("log");
-    if (log) {
-	string icn= Resources::global_res->getIconPath("Log");
-	assert(icn!="");
-	itm->addIcon(icn);
-	itm->setQuickView("Logging enabled");
+    RuleSetList  *pl=(RuleSetList*)get_user_data();
+
+    if (pl->supportsLogging())
+    {
+        bool    log=PolicyRule::cast(rule)->getLogging();
+        if (log) {
+            string icn= Resources::global_res->getIconPath("Log");
+            assert(icn!="");
+            itm->addIcon(icn);
+            itm->setQuickView(_("Logging enabled"));
+        }
+        else {
+            itm->setQuickView(_("Logging disabled"));
+        }
     }
-    else {
-	itm->setQuickView("Logging disabled");
-    }
-    vbox->pack_start( *itm , true , true , 0 );
-    itm->show();
 
-    if (o) itm->set_state(GTK_STATE_SELECTED);
+    addPolicyListItem(itm);
 }
-      
-void PolicyListRuleLog::popup_menu(PolicyListItem *itm)
+
+void PolicyListRuleLog::show_popup_menu(PolicyListItem *itm)
 {
-    gen_popup_menu         *gpm;
-    string icn;
-
-    const char *menu_choices[] = 
-    { "Log",
-      "Do not log",
-      NULL } ;
-    gpm=new gen_popup_menu( menu_choices );
-    gpm->popup(0,0);
-    gint res=gpm->run();
-    delete gpm;
-
-    if (res>=0) {
-	rule->setBool("log", (res==0) );
-	rebuild();
+    create_popup_menu();
+    popup_menu->popup(0,0);
+}
 
+void PolicyListRuleLog::popup_menu_callback(int menu_choice)
+{
+    if (menu_choice>=0) {
+	PolicyRule::cast(rule)->setLogging( (menu_choice==0) );
+	build();
 	set_data_changed_flag(true);
     }
-
 }
+
 /*******************************************************************/
 
 
@@ -882,48 +1069,46 @@ PolicyListRuleComment::PolicyListRuleComment(gint r,gint c, Rule *ru) :
     PolicyListElement(r,c)
 {
     rule=ru;
-    rebuild();
+
+
     selected_item=get_first_child();
+
+    drag_dest_unset();
 }
 
-void PolicyListRuleComment::rebuild() 
+void PolicyListRuleComment::create_popup_menu()
 {
-    bool o=false;
+    if (popup_menu==NULL)
+    {
+        const char *menu_items[] = 
+            { _("Edit Comment"),
+              NULL } ;
 
-    deselect();
-
-    PolicyListItem *itm;
-
-    Gtk::Widget *w=get_first_child();
-    if (w) { o=true; vbox->remove(*w); }
-
-    itm= manage(new PolicyListCommentItem( rule->getComment() ));
-    vbox->pack_start( *itm , true , true , 0 );
-    itm->show();
-
-    if (o) itm->set_state(GTK_STATE_SELECTED);
+        popup_menu=new popupMenu(menu_items);
+        popup_menu->menu_select.connect(SigC::slot(this,&PolicyListRuleComment::popup_menu_callback));
+    }
 }
-      
-void PolicyListRuleComment::popup_menu(PolicyListItem *itm)
+
+void PolicyListRuleComment::build() 
 {
-    gen_popup_menu         *gpm;
-    //PolicyList             *pl=(PolicyList*)get_user_data();
+    removePolicyListItem();
+    addPolicyListItem(new PolicyListCommentItem( rule->getComment() ));
+}
 
-    const char *menu_choices[] = 
-    { "Edit Comment",
-      NULL } ;
+void PolicyListRuleComment::show_popup_menu(PolicyListItem *itm)
+{
+    create_popup_menu();
+    popup_menu->popup(0,0);
+}
 
-    gpm=new gen_popup_menu( menu_choices );
-    gpm->popup(0,0);
-    int res=gpm->run();
-    delete gpm;
-
-    if (res==0) {
+void PolicyListRuleComment::popup_menu_callback(int menu_choice)
+{
+    if (menu_choice==0) {
 	CommentDialog *cmdlg=new CommentDialog( rule->getComment() );
 	string str=cmdlg->run();
 	delete cmdlg;
 	rule->setComment(str);
-	rebuild();
+	build();
 	set_data_changed_flag(true);
     }
 }
@@ -934,54 +1119,70 @@ PolicyListRuleDir::PolicyListRuleDir(gint r,gint c, Rule *ru) :
     PolicyListElement(r,c)
 {
     rule=ru;
-    rebuild();
+
+
     selected_item=get_first_child();
+
+    drag_dest_unset();
 }
 
-void PolicyListRuleDir::rebuild() 
+void PolicyListRuleDir::create_popup_menu()
 {
-    bool o=false;
+    if (popup_menu==NULL)
+    {
+        const char *menu_items[] = 
+            { _("Inbound"),
+              _("Outbound"),
+              _("Both"),
+              NULL } ;
 
-    deselect();
+        popup_menu=new popupMenu(menu_items);
+        popup_menu->menu_select.connect(SigC::slot(this,&PolicyListRuleDir::popup_menu_callback));
+    }
+}
 
-    PolicyListItem *itm;
+void PolicyListRuleDir::build() 
+{
+    PolicyRule     *pr=PolicyRule::cast(rule);
 
-    Gtk::Widget *w=get_first_child();
-    if (w) { o=true; vbox->remove(*w); }
+    removePolicyListItem();
 
-    itm= manage(new PolicyListItem( "","" ));
+    if (pr->getDirection()==PolicyRule::Undefined) 
+        pr->setDirection(PolicyRule::Both);
 
-    string dir=rule->getStr("direction");
+    string dir= pr->getDirectionAsString();
     string icn=	Resources::global_res->getIconPath(dir);
 
-    itm->addIcon(icn);
-    itm->addLabel(dir);
-
-    vbox->pack_start( *itm , true , true , 0 );
-    itm->show();
-
-    if (o) itm->set_state(GTK_STATE_SELECTED);
+    addPolicyListItem(new PolicyListItem( icn,"",dir ));
 }
-      
-void PolicyListRuleDir::popup_menu(PolicyListItem *itm)
+
+void PolicyListRuleDir::show_popup_menu(PolicyListItem *itm)
 {
-    gen_popup_menu         *gpm;
+    create_popup_menu();
+    popup_menu->popup(0,0);
+}
 
-    const char *menu_choices[] = 
-    { "Inbound",
-      "Outbound",
-      "Both",
-      NULL } ;
+void PolicyListRuleDir::popup_menu_callback(int menu_choice)
+{
+/* direction makes sence only for PolicyRule */
+    PolicyRule *prule=PolicyRule::cast(rule);
+    assert(prule!=NULL);
 
-    gpm=new gen_popup_menu( menu_choices );
-    gpm->popup(0,0);
-    gint res=gpm->run();
-    delete gpm;
-
-    if (res>=0) {
-	rule->setStr("direction", menu_choices[res] );
-	rebuild();
+    if (menu_choice>=0) {
+        switch (menu_choice)
+        {
+        case 0:
+            prule->setDirection( PolicyRule::Inbound );
+            break;
+        case 1:
+            prule->setDirection( PolicyRule::Outbound );
+            break;
+        case 2:
+            prule->setDirection( PolicyRule::Both );
+            break;
+        }
 
+	build();
 	set_data_changed_flag(true);
     }
 }
@@ -993,85 +1194,102 @@ PolicyListRuleOpt::PolicyListRuleOpt(gint r,gint c, Rule *ru) :
     PolicyListElement(r,c)
 {
     rule=ru;
-    rebuild();
+
+
     selected_item=get_first_child();
+
+    drag_dest_unset();
 }
 
-void PolicyListRuleOpt::rebuild() 
+void PolicyListRuleOpt::create_popup_menu()
 {
-    bool o=false;
+    if (popup_menu==NULL)
+    {
+        const char *menu_items[] = 
+            { _("Modify Options"),
+              "",
+              _("Turn logging ON"),
+              _("Turn logging OFF"),
+              NULL } ;
 
-    deselect();
+        popup_menu=new popupMenu(menu_items);
+        popup_menu->menu_select.connect(SigC::slot(this,&PolicyListRuleOpt::popup_menu_callback));
+    }
+}
 
+void PolicyListRuleOpt::build() 
+{
     PolicyListItem *itm;
 
-    Gtk::Widget *w=get_first_child();
-    if (w) { o=true; vbox->remove(*w); }
+    removePolicyListItem();
 
-    itm= manage(new PolicyListItem( "","" ));
+    itm= new PolicyListItem( "","","" );
 
     string qv_text;
     string icn;
 
-    itm->clear();
+//    itm->clear();
+    RuleSetList  *pl=(RuleSetList*)get_user_data();
 
-    bool    log=rule->getBool("log");
-    if (log) {
-	icn= Resources::global_res->getIconPath("Log");
-	assert(icn!="");
-	itm->addIcon(icn);
-	qv_text+="Logging enabled";
-    }
-    else {
-	qv_text+="Logging disabled";
+    if (pl->supportsLogging())
+    {
+        bool    log=rule->getBool("log");
+        if (log) {
+            icn= Resources::global_res->getIconPath("Log");
+            assert(icn!="");
+            itm->addIcon(icn);
+
+            qv_text+=_("Logging enabled");
+        }
+        else {
+            qv_text+=_("Logging disabled");
+        }
     }
 
-    if (! rule->getOptionsObject()->isDefault()) {
-	icn=Resources::global_res->getIconPath("Options");
-	assert(icn!="");
-	itm->addIcon(icn);
-	qv_text+="\nNon-default options";
+    if (pl->supportsRuleOptions())
+    {
+        if (! isDefaultOptions( rule->getOptionsObject() )) {
+            icn=Resources::global_res->getIconPath("Options");
+            assert(icn!="");
+            itm->addIcon(icn);
+            qv_text+=_("\nNon-default options");
+        }
     }
-
     itm->setQuickView( qv_text );
 
-    vbox->pack_start( *itm , true , true , 0 );
-    itm->show();
-
-    if (o) itm->set_state(GTK_STATE_SELECTED);
+    addPolicyListItem(itm);
 }
-      
-void PolicyListRuleOpt::popup_menu(PolicyListItem *itm)
+
+void PolicyListRuleOpt::show_popup_menu(PolicyListItem *itm)
 {
-    gen_popup_menu         *gpm;
-    string icn;
+    create_popup_menu();
 
-    const char *menu_choices[] = 
-    { "Modify Options",
-      "",
-      "Turn logging ON",
-      "Turn logging OFF",
-      NULL } ;
+    RuleSetList  *pl=(RuleSetList*)get_user_data();
 
-    gpm=new gen_popup_menu( menu_choices );
-    gpm->popup(0,0);
-    gint res=gpm->run();
-    delete gpm;
+    popup_menu->set_sensitive(0,pl->supportsRuleOptions());
+    popup_menu->set_sensitive(2,pl->supportsLogging());
+    popup_menu->set_sensitive(3,pl->supportsLogging());
 
-    if (res>=0) {
-	switch (res) {
+    popup_menu->popup(0,0);
+}
+
+void PolicyListRuleOpt::popup_menu_callback(int menu_choice)
+{
+    if (menu_choice>=0) {
+	switch (menu_choice) {
 	case 0: 
 	{
 	    RuleOptionsDialog *dlg=new RuleOptionsDialog(rule->getOptionsObject());
 	    dlg->run();
 	    delete dlg;
-	    rebuild();
+	    build();
 	    break;
 	}
 	case 2:
 	case 3:
-	    rule->setBool("log", (res==2) );
-	    rebuild();
+            PolicyRule *pr=PolicyRule::cast(rule);
+            if (pr) pr->setLogging( (menu_choice==2) );
+	    build();
 	    break;
 	}
 
@@ -1082,9 +1300,3 @@ void PolicyListRuleOpt::popup_menu(PolicyListItem *itm)
 
 
 
-
-
-
-
-
-
