@@ -2,11 +2,11 @@
 
                           Firewall Builder
 
-                 Copyright (C) 2001 Vadim Kurland
+                 Copyright (C) 2001 NetCitadel, LLC
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: SimpleProgressDialog.cc,v 1.1 2001/12/19 06:47:43 vkurland Exp $
+  $Id: SimpleProgressDialog.cc,v 1.3 2003/03/04 05:40:05 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -32,7 +32,12 @@ SimpleProgressDialog::SimpleProgressDialog() {}
 
 void SimpleProgressDialog::setTitle(const string &t)
 {
-    title->set_text(t);
+    set_title(t);
+}
+
+void SimpleProgressDialog::setText(const string &t)
+{
+    text->set_text(t);
 }
 
 void SimpleProgressDialog::setProgressBarRange(gfloat min,gfloat max)
