@@ -2,11 +2,11 @@
 
                           Firewall Builder
 
-                 Copyright (C) 2000 Vadim Kurland
+                 Copyright (C) 2000 NetCitadel, LLC
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: StandardRulesDruid2.cc,v 1.20 2001/12/19 11:47:37 lord Exp $
+  $Id: StandardRulesDruid2.cc,v 1.36 2003/09/21 03:44:39 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -24,30 +24,30 @@
 
 */
 
+#include "config.h"
  
-#include "StandardRulesDruid.hh"
-
-#include "fwbuilder/FWObjectDatabase.hh"
 #include "fwbuilder/Interface.hh"
 #include "fwbuilder/Host.hh"
 #include "fwbuilder/Firewall.hh"
-
-#include "BuiltinDialog.hh"
-
 #include "fwbuilder/Policy.hh"
 #include "fwbuilder/InterfacePolicy.hh"
 #include "fwbuilder/NAT.hh"
-#include "PolicyList.hh"
 #include "fwbuilder/Rule.hh"
 #include "fwbuilder/RuleElement.hh"
-
 #include "fwbuilder/FWOptions.hh"
 
-#include "main_window.hh"
+#include "StandardRulesDruid.hh"
+#include "FWObjectDatabaseGUI.hh"
+#include "PolicyList.hh"
+#include "BuiltinDialog.hh"
+
+
 #include "helpers.hh"
 
 #include <gtk--.h>
 
+#include <assert.h>
+
 
 using namespace libfwbuilder;
 
@@ -56,22 +56,10 @@ using namespace libfwbuilder;
  */
 Interface* StandardRulesDruid::findLoopbackInterface()
 {
-    FWObject *loopback;
-    string    name;
-    string    addr;
-    FWObject *o;
-    vector<FWObject*>::iterator m;
-    for (m=fw_object->begin(); m!=fw_object->end(); ++m) {
-	if ( (o=(*m))!=NULL  && Interface::isA(o) ) {
-	    name=o->getName();
-	    addr=o->getStr("address");
-	    if (name=="lo0" || name=="loopback0" ||
-		name=="Lo0" || name=="Loopback0" ||
-		addr=="127.0.0.1" ) {
-		loopback=o;
-		return (Interface*)loopback;
-	    } 
-	}
+    list<FWObject*> l=fw_object->getByType(Interface::TYPENAME);
+    for (list<FWObject*>::iterator m=l.begin(); m!=l.end(); m++) {
+	if ( Interface::cast(*m)->isLoopback() ) 
+	    return Interface::cast(*m);
     }
     return NULL;
 }
@@ -81,23 +69,10 @@ Interface* StandardRulesDruid::findLoopbackInterface()
  */
 Interface* StandardRulesDruid::findNonLoopbackInterface()
 {
-    string    name;
-    string    addr;
-    FWObject *o;
-    vector<FWObject*>::iterator m;
-    for (m=fw_object->begin(); m!=fw_object->end(); ++m) {
-	if ( (o=(*m))!=NULL  && Interface::isA(o) ) {
-	    name=o->getName();
-	    addr=o->getStr("address");
-	    if (name=="lo0" || name=="loopback0" ||
-		name=="Lo0" || name=="Loopback0" ||
-		addr=="127.0.0.1" ) {
-
-		continue;
-	    } else 
-		return (Interface*)o;
-
-	}
+    list<FWObject*> l=fw_object->getByType(Interface::TYPENAME);
+    for(list<FWObject*>::iterator m=l.begin(); m!=l.end(); m++) {
+	if ( ! Interface::cast(*m)->isLoopback() ) 
+	    return Interface::cast(*m);
     }
     return NULL;
 }
@@ -107,7 +82,7 @@ void StandardRulesDruid::generateRulesForHostProtection()
 {
     Policy      *pol=((Firewall*)fw_object)->getPolicy();
     InterfacePolicy *ipol, *loipol;
-    Rule        *rule;
+    PolicyRule  *rule;
     RuleElement *rel;
     FWObject    *ifs, *loopback, *smtp, *http, *ssh, *ftp, *telnet, *icmp;
 
@@ -142,69 +117,63 @@ void StandardRulesDruid::generateRulesForHostProtection()
 	p30->allow_telnet->get_active() ||
 	p30->allow_icmp->get_active()  ) {
 
-	smtp=_findObjectByName(FWObjectDatabase::db->std.ServicesId ,"smtp" );
-	http=_findObjectByName(FWObjectDatabase::db->std.ServicesId ,"http" );
-	ssh=_findObjectByName(FWObjectDatabase::db->std.ServicesId , "ssh" );
-	ftp=_findObjectByName(FWObjectDatabase::db->std.ServicesId , "ftp" );
-	telnet=_findObjectByName(FWObjectDatabase::db->std.ServicesId ,"telnet");
-	icmp=_findObjectByName(FWObjectDatabase::db->std.ServicesId ,"all_icmp_unreachables");
+	smtp=_findObjectByName(FWObjectDatabaseGUI::getServicesId() ,"smtp" );
+	http=_findObjectByName(FWObjectDatabaseGUI::getServicesId() ,"http" );
+	ssh=_findObjectByName(FWObjectDatabaseGUI::getServicesId() , "ssh" );
+	ftp=_findObjectByName(FWObjectDatabaseGUI::getServicesId() , "ftp" );
+	telnet=_findObjectByName(FWObjectDatabaseGUI::getServicesId() ,"telnet");
+	icmp=_findObjectByName(FWObjectDatabaseGUI::getServicesId() ,"all_icmp_unreachables");
 
 	rule= pol->insertRuleAtTop();
-	rule->setStr("action","Accept");
-	rule->setBool("log",false);
-	rel=RuleElement::cast( rule->getFirstByType("Srv") );
+	rule->setAction(PolicyRule::Accept);
+	rule->setLogging(false);
+	rel=rule->getSrv();
 
 	if (p30->allow_smtp->get_active() && smtp ) {
-	    if (rel->isAny())  rel->clearChildren();
 	    rel->addRef(smtp);
 	}
 
 	if (p30->allow_http->get_active() && http ) {
-	    if (rel->isAny())  rel->clearChildren();
 	    rel->addRef(http);
 	}
 
 	if (p30->allow_ssh->get_active() && ssh ) {
-	    if (rel->isAny())  rel->clearChildren();
 	    rel->addRef(ssh);
 	}
 
 	if (p30->allow_ftp->get_active() && ftp ) {
-	    if (rel->isAny())  rel->clearChildren();
 	    rel->addRef(ftp);
 	}
 
 	if (p30->allow_telnet->get_active() && telnet ) {
-	    if (rel->isAny())  rel->clearChildren();
 	    rel->addRef(telnet);
 	}
 
 	if (p30->allow_icmp->get_active() && icmp ) {
-	    if (rel->isAny())  rel->clearChildren();
 	    rel->addRef(icmp);
 	}
 
-	rel=RuleElement::cast( rule->getFirstByType("Dst") );
-	if (rel->isAny())  rel->clearChildren();
+	rel=rule->getDst();
 	rel->addRef(fw_object);
     }
 
     if (p30->short_fragments->get_active()) {
 
-	FWObject *o=_findObjectByName(FWObjectDatabase::db->std.ServicesId  , 
+	FWObject *o=_findObjectByName(FWObjectDatabaseGUI::getServicesId()  , 
 			    "ip_fragments" );
 	if (o) {
 	    rule= pol->insertRuleAtTop();
-	    rule->setStr("action","Deny");
-	    rule->setBool("log",true);
-	    rule->setComment("block fragments");
+	    rule->setAction(PolicyRule::Deny);
+	    rule->setLogging(true);
 
-	    rel=RuleElement::cast( rule->getFirstByType("Srv") );
-	    if (rel->isAny())  rel->clearChildren();
+	    rule->setComment(_("block fragments"));
+
+
+
+	    rel=rule->getSrv();
 	    rel->addRef(o);
 
-	    rel=RuleElement::cast( rule->getFirstByType("Dst") );
-	    if (rel->isAny())  rel->clearChildren();
+	    rel=rule->getDst();
 	    rel->addRef(fw_object);
 
 	    FWOptions* opt=rule->getOptionsObject();
@@ -218,39 +187,28 @@ void StandardRulesDruid::generateRulesForHostProtection()
 
     if (p30->allow_all_on_loopback->get_active() && loipol!=NULL) {
 	rule= loipol->appendRuleAtBottom();
-	rule->setStr("action","Accept");
-	rule->setStr("direction","Inbound");
-	rule->setBool("log",false);
-	rule->setComment("allow everything on loopback");
-	rel=RuleElement::cast( rule->getFirstByType("Dst") );
-	if (rel->isAny())  rel->clearChildren();
-	rel->addRef(fw_object);
+	rule->setAction(PolicyRule::Accept);
+	rule->setDirection(PolicyRule::Both);
+	rule->setLogging(false);
+
+	rule->setComment(_("allow everything on loopback"));
+
+
 
 	FWOptions* opt=rule->getOptionsObject();
-	if (opt)
-	    opt->setBool("stateless",true);
-
-	rule= loipol->appendRuleAtBottom();
-	rule->setStr("action","Accept");
-	rule->setStr("direction","Outbound");
-	rule->setBool("log",false);
-	rule->setComment("allow everything on loopback");
-	rel=RuleElement::cast( rule->getFirstByType("Src") );
-	if (rel->isAny())  rel->clearChildren();
-	rel->addRef(fw_object);
-
-	opt=rule->getOptionsObject();
-	if (opt)
-	    opt->setBool("stateless",true);
+	assert (opt!=NULL);
+	opt->setBool("stateless",true);
     }
 
     if (p30->allow_all_outgoing->get_active() ) {
 	rule= pol->appendRuleAtBottom();
-	rule->setStr("action","Accept");
-	rule->setBool("log",false);
-	rule->setComment("allow all outgoing connections");
-	rel=RuleElement::cast( rule->getFirstByType("Src") );
-	if (rel->isAny())  rel->clearChildren();
+	rule->setAction(PolicyRule::Accept);
+	rule->setLogging(false);
+
+	rule->setComment(_("allow all outgoing connections"));
+
+
+	rel=rule->getSrc();
 	rel->addRef(fw_object);
     }
 
@@ -259,16 +217,18 @@ void StandardRulesDruid::generateRulesForHostProtection()
 
     if (p30->catch_all->get_active() && pol!=NULL) {
 	rule= pol->appendRuleAtBottom();
-	rule->setStr("action","Deny");
-	rule->setBool("log",true);
-	rule->setComment("'catch all' rule");
-	rel=RuleElement::cast( rule->getFirstByType("Dst") );
-	if (rel->isAny())  rel->clearChildren();
+	rule->setAction(PolicyRule::Deny);
+	rule->setLogging(true);
+
+	rule->setComment(_("'catch all' rule"));
+
+
+	rel=rule->getDst();
 	rel->addRef(fw_object);
 
-	FWOptions* opt=rule->getOptionsObject();
-	if (opt)
-	    opt->setBool("stateless",true);
+//	FWOptions* opt=rule->getOptionsObject();
+//	if (opt)
+//	    opt->setBool("stateless",true);
     }
 }
 
@@ -278,9 +238,16 @@ void StandardRulesDruid::generateRulesForNetProtection()
     Policy      *pol=((Firewall*)fw_object)->getPolicy();
     NAT         *nat=((Firewall*)fw_object)->getNAT();
     InterfacePolicy *ipol, *loipol;
-    Rule        *rule;
+    PolicyRule  *rule;
+    NATRule     *nrule;
     RuleElement *rel;
     FWObject    *ifs, *o, *loopback;
+    FWObject    *ssh, *dns, *dhcp, *bcast;
+
+    ssh   =_findObjectByName(FWObjectDatabaseGUI::getServicesId() , "ssh"    );
+    dns   =_findObjectByName(FWObjectDatabaseGUI::getServicesId() , "DNS"    );
+    dhcp  =_findObjectByName(FWObjectDatabaseGUI::getServicesId() , "DHCP"   );
+    bcast =_findObjectByName(FWObjectDatabaseGUI::getAddressRangesId() , "broadcast" );
 
 /* 
  *   Interface chosen on the page ASDruidP11Dialog
@@ -298,16 +265,21 @@ void StandardRulesDruid::generateRulesForNetProtection()
 /*************************************************************/
 
 
-    if (p12->anti_spoof_out->get_active()) {
-
+    if (p12->anti_spoof_out->get_active()) 
+    {
 	rule= ipol->insertRuleAtTop();
-	rule->setStr("direction","Outbound");
-	rule->setStr("action","Deny");
-	rule->setBool("log",true);
-	rule->setComment("Anti-spoofing rule");
 
-	rel=RuleElement::cast( rule->getFirstByType("Src") );
-	if (rel->isAny())  rel->clearChildren();
+        rule->setDirection( PolicyRule::Outbound );
+
+
+	rule->setAction(PolicyRule::Deny);
+	rule->setLogging(true);
+
+	rule->setComment(_("Anti-spoofing rule"));
+
+
+
+	rel=rule->getSrc();
 	_fillRuleElementP20(rel);
 	rel->addRef(fw_object);          /* firewall itself */
 	rel->setNeg(true);
@@ -319,16 +291,21 @@ void StandardRulesDruid::generateRulesForNetProtection()
 
 /*************************************************************/
 
-    if (p12->anti_spoof_in->get_active()) {
-
+    if (p12->anti_spoof_in->get_active()) 
+    {
 	rule= ipol->insertRuleAtTop();
-	rule->setStr("direction","Inbound");
-	rule->setStr("action","Deny");
-	rule->setBool("log",true);
-	rule->setComment("Anti-spoofing rule");
 
-	rel=RuleElement::cast( rule->getFirstByType("Src") );
-	if (rel->isAny())  rel->clearChildren();
+        rule->setDirection( PolicyRule::Inbound );
+
+
+	rule->setAction(PolicyRule::Deny);
+	rule->setLogging(true);
+
+	rule->setComment(_("Anti-spoofing rule"));
+
+
+
+	rel=rule->getSrc();
 	_fillRuleElementP20(rel);
 	rel->addRef(fw_object);          /* firewall itself */
 
@@ -339,93 +316,148 @@ void StandardRulesDruid::generateRulesForNetProtection()
 
 /*************************************************************/
 
-    if (p12->short_fragments->get_active()) {
-
-	o=_findObjectByName(FWObjectDatabase::db->std.ServicesId  , 
+    if (p12->short_fragments->get_active()) 
+    {
+	o=_findObjectByName(FWObjectDatabaseGUI::getServicesId()  , 
 			    "ip_fragments" );
 
 	if (o) {
 	    rule= pol->insertRuleAtTop();
-	    rule->setStr("action","Deny");
-	    rule->setBool("log",true);
-	    rule->setComment("block fragments");
-
-	    rel=RuleElement::cast( rule->getFirstByType("Srv") );
-	    if (rel->isAny())  rel->clearChildren();
-	    rel->addRef(o);
-
+	    rule->setAction(PolicyRule::Deny);
+	    rule->setLogging(true);
+	    rel=rule->getSrv();  rel->addRef(o);
 	    FWOptions* opt=rule->getOptionsObject();
-	    if (opt)
-		opt->setBool("stateless",true);
+	    if (opt)  opt->setBool("stateless",true);
+	    rule->setComment(_("block fragments"));
 	}
     }
 
 /*************************************************************/
 
-    if (p12->allow_all_on_loopback->get_active() && loipol!=NULL) {
+    if (p12->allow_all_on_loopback->get_active() && loipol!=NULL) 
+    {
 	rule= loipol->appendRuleAtBottom();
-	rule->setStr("action","Accept");
-	rule->setStr("direction","Inbound");
-	rule->setBool("log",false);
-	rule->setComment("allow everything on loopback");
-	rel=RuleElement::cast( rule->getFirstByType("Dst") );
-	if (rel->isAny())  rel->clearChildren();
-	rel->addRef(fw_object);
+	rule->setAction(PolicyRule::Accept);
+	rule->setDirection(PolicyRule::Both);
+	rule->setLogging(false);
+
+	rule->setComment(_("allow everything on loopback"));
 
 	FWOptions* opt=rule->getOptionsObject();
-	if (opt)
-	    opt->setBool("stateless",true);
-
-	rule= loipol->appendRuleAtBottom();
-	rule->setStr("action","Accept");
-	rule->setStr("direction","Outbound");
-	rule->setBool("log",false);
-	rule->setComment("allow everything on loopback");
-	rel=RuleElement::cast( rule->getFirstByType("Src") );
-	if (rel->isAny())  rel->clearChildren();
-	rel->addRef(fw_object);
-
-	opt=rule->getOptionsObject();
-	if (opt)
-	    opt->setBool("stateless",true);
+	assert (opt!=NULL);
+	opt->setBool("stateless",true);
     }
 
-
-    if (p12->masquerading->get_active()) {
+    if (p12->ssh_from_anywhere->get_active() && ssh)
+    {
 	rule= pol->appendRuleAtBottom();
-	rule->setStr("action","Accept");
-	rule->setBool("log",false);
-	rule->setComment("'masquerading' rule");
+	rule->setAction(PolicyRule::Accept);
+	rule->setLogging(false);
+	rel=rule->getDst();   rel->addRef( fw_object );
+        rel=rule->getSrv();   rel->addRef(ssh);
+	rule->setComment(_("ssh access to firewall"));
+    }
 
-	rel=RuleElement::cast( rule->getFirstByType("Src") );
-	if (rel->isAny())  rel->clearChildren();
-	_fillRuleElementP20(rel);
+    if (p12->ssh_from_lan->get_active() && ssh)
+    {
+	rule= pol->appendRuleAtBottom();
+	rule->setAction(PolicyRule::Accept);
+	rule->setLogging(false);
+        rel=rule->getSrc();  _fillRuleElementP20(rel);
+	rel=rule->getDst();  rel->addRef( fw_object );
+        rel=rule->getSrv();  rel->addRef(ssh);
+	rule->setComment(_("ssh access to firewall"));
+    }
+
+    if (p12->uses_dns_on_lan->get_active() && dns)
+    {
+	rule= pol->appendRuleAtBottom();
+	rule->setAction(PolicyRule::Accept);
+	rule->setLogging(false);
+        rel=rule->getSrc();   rel->addRef( fw_object );
+	rel=rule->getDst();   _fillRuleElementP20(rel);
+        rel=rule->getSrv();   rel->addRef(dns);
+	rule->setComment(_("firewall uses DNS server on LAN"));
+    }
+
+    if (p12->uses_dns_on_inet->get_active() && dns)
+    {
+	rule= pol->appendRuleAtBottom();
+	rule->setAction(PolicyRule::Accept);
+	rule->setLogging(false);
+        rel=rule->getSrc();   rel->addRef( fw_object );
+        rel=rule->getSrv();   rel->addRef(dns);
+	rule->setComment(_("firewall uses DNS server on Inet"));
+    }
+
+    if (p12->provides_dns_for_lan->get_active() && dns)
+    {
+	rule= pol->appendRuleAtBottom();
+	rule->setAction(PolicyRule::Accept);
+	rule->setLogging(false);
+        rel=rule->getSrc();   _fillRuleElementP20(rel);
+	rel=rule->getDst();   rel->addRef( fw_object );
+        rel=rule->getSrv();   rel->addRef(dns);
+	rule->setComment(_("firewall serves as DNS server for LAN"));
+    }
+
+    if (p12->provides_dhcp->get_active() && dhcp && bcast)
+    {
+	rule= pol->appendRuleAtBottom();
+	rule->setAction(PolicyRule::Accept);
+	rule->setLogging(false);
+        rel=rule->getSrc();   _fillRuleElementP20(rel);
+	rel=rule->getDst();   rel->addRef( fw_object ); rel->addRef( bcast );
+        rel=rule->getSrv();   rel->addRef(dhcp);
+	rule->setComment(_("firewall serves as DHCP server for LAN"));
+
+	rule= pol->appendRuleAtBottom();
+	rule->setAction(PolicyRule::Accept);
+	rule->setLogging(false);
+	rel=rule->getSrc();   rel->addRef( fw_object );
+        rel=rule->getDst();   _fillRuleElementP20(rel);
+        rel=rule->getSrv();   rel->addRef(dhcp);
+	rule->setComment(_("firewall serves as DHCP server for LAN"));
+    }
+
+    if (p12->masquerading->get_active()) 
+    {
+	rule= pol->appendRuleAtBottom();
+	rule->setAction(PolicyRule::Accept);
+	rule->setLogging(false);
+	rel=rule->getSrc();	_fillRuleElementP20(rel);
 
 /*  add rule to NAT policy */
-	rule= nat->insertRuleAtTop();
-	rel=RuleElement::cast( rule->getFirstByType("OSrc") );
-	if (rel->isAny())  rel->clearChildren();
-	_fillRuleElementP20(rel);
-	rel=RuleElement::cast( rule->getFirstByType("TSrc") );
-	if (rel->isAny())  rel->clearChildren();
-	rel->addRef(fw_object);   /* firewall itself as masq. address */
+	nrule= nat->insertRuleAtTop();
+	rel=nrule->getOSrc();	_fillRuleElementP20(rel);
+	rel=nrule->getTSrc();	
 
+	list<FWObject*> l2=fw_object->getByType(Interface::TYPENAME);
+	for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i)
+        {
+	    Interface *iface=dynamic_cast<Interface*>(*i);
+	    assert(iface);
+
+	    if ( iface->isExt() ) 
+                rel->addRef(iface);   /* external interface as masq. address */
+	}
+
+	rule->setComment(_("'masquerading' rule"));
     }
 
 /*************************************************************/
 
-    if (p12->catch_all->get_active()) {
+    if (p12->catch_all->get_active()) 
+    {
 	rule= pol->appendRuleAtBottom();
-	rule->setStr("action","Deny");
-	rule->setBool("log",true);
-	rule->setComment("'catch all' rule");
+	rule->setAction(PolicyRule::Deny);
+	rule->setLogging(true);
 
-	FWOptions* opt=rule->getOptionsObject();
-	if (opt)
-	    opt->setBool("stateless",true);
+//	FWOptions* opt=rule->getOptionsObject();
+//        assert(opt!=NULL);
+//	opt->setBool("stateless",true);
+	rule->setComment(_("'catch all' rule"));
     }
-
-
 }
 
 void StandardRulesDruid::generateRulesForNetAndDMZProtection()
@@ -433,7 +465,8 @@ void StandardRulesDruid::generateRulesForNetAndDMZProtection()
     Policy      *pol=((Firewall*)fw_object)->getPolicy();
     NAT         *nat=((Firewall*)fw_object)->getNAT();
     InterfacePolicy *ipol, *dmzipol, *loipol;
-    Rule        *rule;
+    PolicyRule  *rule;
+    NATRule     *nrule;
     RuleElement *rel;
     FWObject    *ifs, *dmzifs, *o, *loopback;
 
@@ -461,16 +494,20 @@ void StandardRulesDruid::generateRulesForNetAndDMZProtection()
 /*************************************************************/
 
 
-    if (p47->anti_spoof_out->get_active()) {
-
+    if (p47->anti_spoof_out->get_active()) 
+    {
 	rule= ipol->insertRuleAtTop();
-	rule->setStr("direction","Outbound");
-	rule->setStr("action","Deny");
-	rule->setBool("log",true);
-	rule->setComment("Anti-spoofing rule");
 
-	rel=RuleElement::cast( rule->getFirstByType("Src") );
-	if (rel->isAny())  rel->clearChildren();
+        rule->setDirection( PolicyRule::Outbound );
+
+	rule->setAction(PolicyRule::Deny);
+	rule->setLogging(true);
+
+	rule->setComment(_("Anti-spoofing rule"));
+
+
+
+	rel=rule->getSrc();
 	_fillRuleElementP49net(rel);
 	_fillRuleElementP49dmz(rel);
 	rel->addRef(fw_object);          /* firewall itself */
@@ -483,16 +520,20 @@ void StandardRulesDruid::generateRulesForNetAndDMZProtection()
 
 /*************************************************************/
 
-    if (p47->anti_spoof_in->get_active()) {
-
+    if (p47->anti_spoof_in->get_active()) 
+    {
 	rule= ipol->insertRuleAtTop();
-	rule->setStr("direction","Inbound");
-	rule->setStr("action","Deny");
-	rule->setBool("log",true);
-	rule->setComment("Anti-spoofing rule");
 
-	rel=RuleElement::cast( rule->getFirstByType("Src") );
-	if (rel->isAny())  rel->clearChildren();
+        rule->setDirection( PolicyRule::Inbound );
+
+	rule->setAction(PolicyRule::Deny);
+	rule->setLogging(true);
+
+	rule->setComment(_("Anti-spoofing rule"));
+
+
+
+	rel=rule->getSrc();
 	_fillRuleElementP49net(rel);
 	_fillRuleElementP49dmz(rel);
 	rel->addRef(fw_object);          /* firewall itself */
@@ -504,18 +545,22 @@ void StandardRulesDruid::generateRulesForNetAndDMZProtection()
 
 /*************************************************************/
 
-    if (p47->short_fragments->get_active()) {
+    if (p47->short_fragments->get_active()) 
+    {
 
-	o=_findObjectByName(FWObjectDatabase::db->std.ServicesId  , 
+	o=_findObjectByName(FWObjectDatabaseGUI::getServicesId()  , 
 			    "ip_fragments" );
-	if (o) {
+	if (o) 
+        {
 	    rule= pol->insertRuleAtTop();
-	    rule->setStr("action","Deny");
-	    rule->setBool("log",true);
-	    rule->setComment("block fragments");
+	    rule->setAction(PolicyRule::Deny);
+	    rule->setLogging(true);
 
-	    rel=RuleElement::cast( rule->getFirstByType("Srv") );
-	    if (rel->isAny())  rel->clearChildren();
+	    rule->setComment(_("block fragments"));
+
+
+
+	    rel=rule->getSrv();
 	    rel->addRef(o);
 
 	    FWOptions* opt=rule->getOptionsObject();
@@ -526,73 +571,93 @@ void StandardRulesDruid::generateRulesForNetAndDMZProtection()
 
 /*************************************************************/
 
-    if (p47->allow_all_on_loopback->get_active() && loipol!=NULL) {
+    if (p47->allow_all_on_loopback->get_active() && loipol!=NULL) 
+    {
 	rule= loipol->appendRuleAtBottom();
-	rule->setStr("action","Accept");
-	rule->setStr("direction","Inbound");
-	rule->setBool("log",false);
-	rule->setComment("allow everything on loopback");
-	rel=RuleElement::cast( rule->getFirstByType("Dst") );
-	if (rel->isAny())  rel->clearChildren();
-	rel->addRef(fw_object);
+	rule->setAction(PolicyRule::Accept);
+	rule->setDirection(PolicyRule::Both);
+	rule->setLogging(false);
+
+	rule->setComment(_("allow everything on loopback"));
+
+
 
 	FWOptions* opt=rule->getOptionsObject();
-	if (opt)
-	    opt->setBool("stateless",true);
-
-	rule= loipol->appendRuleAtBottom();
-	rule->setStr("action","Accept");
-	rule->setStr("direction","Outbound");
-	rule->setBool("log",false);
-	rule->setComment("allow everything on loopback");
-	rel=RuleElement::cast( rule->getFirstByType("Src") );
-	if (rel->isAny())  rel->clearChildren();
-	rel->addRef(fw_object);
-
-	opt=rule->getOptionsObject();
-	if (opt)
-	    opt->setBool("stateless",true);
+	assert (opt!=NULL);
+	opt->setBool("stateless",true);
     }
 
 
 /*************************************************************/
 
-    if (p47->masquerading->get_active()) {
-	rule= pol->appendRuleAtBottom();
-	rule->setStr("action","Accept");
-	rule->setBool("log",false);
-	rule->setComment("'masquerading' rule");
+    if (p47->masquerading->get_active()) 
+    {
+/* first add a rule blocking access to internal net and fw from DMZ */
 
-	rel=RuleElement::cast( rule->getFirstByType("Src") );
-	if (rel->isAny())  rel->clearChildren();
+	rule= pol->appendRuleAtBottom();
+	rule->setAction(PolicyRule::Deny);
+	rule->setLogging(false);
+
+	rule->setComment(
+_("blocking access to internal net\nand firewall from DMZ.\nRules that permit access to servers\non DMZ should be added above.")
+);
+
+	rel=rule->getSrc();
+	_fillRuleElementP49dmz(rel);
+
+	rel=rule->getDst();
+	_fillRuleElementP49net(rel);
+	rel->addRef(fw_object);          /* firewall itself */
+
+
+/* then add masquerading rule */
+
+	rule= pol->appendRuleAtBottom();
+	rule->setAction(PolicyRule::Accept);
+	rule->setLogging(false);
+
+	rule->setComment(_("'masquerading' rule"));
+
+	rel=rule->getSrc();
 	_fillRuleElementP49net(rel);
 	_fillRuleElementP49dmz(rel);
 
 /*  add rule to NAT policy */
-	rule= nat->insertRuleAtTop();
+	nrule= nat->insertRuleAtTop();
 
-	rel=RuleElement::cast( rule->getFirstByType("OSrc") );
-	if (rel->isAny())  rel->clearChildren();
+	rel=nrule->getOSrc();
 	_fillRuleElementP49net(rel);
 
 	if (p40->dmz_needs_nat->get_active())
 	    _fillRuleElementP49dmz(rel);
 
-	rel=RuleElement::cast( rule->getFirstByType("TSrc") );
-	if (rel->isAny())  rel->clearChildren();
-	rel->addRef(fw_object);          /* firewall itself as masq. address */
+	rel=nrule->getTSrc();
+
+	list<FWObject*> l2=fw_object->getByType(Interface::TYPENAME);
+	for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i) 
+        {
+	    Interface *iface=dynamic_cast<Interface*>(*i);
+	    assert(iface);
+
+	    if ( iface->isExt() ) 
+                rel->addRef(iface);   /* external interface as masq. address */
+	}
     }
 
 /*************************************************************/
 
-    if (p47->catch_all->get_active()) {
+    if (p47->catch_all->get_active()) 
+    {
 	rule= pol->appendRuleAtBottom();
-	rule->setStr("action","Deny");
-	rule->setBool("log",true);
-	rule->setComment("'catch all' rule");
-	FWOptions* opt=rule->getOptionsObject();
-	if (opt)
-	    opt->setBool("stateless",true);
+	rule->setAction(PolicyRule::Deny);
+	rule->setLogging(true);
+
+	rule->setComment(_("'catch all' rule"));
+
+
+//	FWOptions* opt=rule->getOptionsObject();
+//	if (opt)
+//	    opt->setBool("stateless",true);
     }
 
 
