@@ -2,11 +2,11 @@
 
                           Firewall Builder
 
-                 Copyright (C) 2000 Vadim Kurland
+                 Copyright (C) 2000 NetCitadel, LLC
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: main_window.cc,v 1.85 2001/12/29 10:06:36 vkurland Exp $
+  $Id: main_window.cc,v 1.136 2003/06/29 06:38:25 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -31,22 +31,28 @@
 #include <cstdlib>
 #include <vector>
 
+#include <assert.h>
+
 #include <gtk--.h>
 
 #include <gdk/gdkkeysyms.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
 #include "glademm_support.hh"
 
 
 #include "main_window.hh"
 #include "helpers.hh"
+#include "SimplePixmap.hh"
 
 #include "fwbuilder/Firewall.hh"
 #include "fwbuilder/Policy.hh"
 #include "fwbuilder/NAT.hh"
+#include "fwbuilder/FWOptions.hh"
+#include "fwbuilder/Constants.hh"
+#include "fwbuilder/Management.hh"
 
 #include "FWObjectDatabaseGUI.hh"
-#include "fwbuilder/Network.hh"
 #include "DialogPlugin.hh"
 #include "PrintDialog.hh"
 #include "PolicyDialog.hh"
@@ -58,35 +64,40 @@
 #include "OptionsDlg.hh"
 #include "ExecDialog.hh"
 #include "MessageDialog.hh"
-#include "fwbuilder/FWOptions.hh"
-
-#include "fwbuilder/Constants.hh"
+#include "FileSel.hh"
 
 #include "StandardRulesDruid.hh"
 #include "DiscoveryDruid.hh"
 
 using namespace libfwbuilder;
 
+std::list<main_window*> main_window::windows;
+
 
 main_window::main_window() :  Gtk::Window(GTK_WINDOW_TOPLEVEL)
 {
 
+    windows.push_back(this);
+
+
     custom_item=NULL;
     policy_in_right_pane=false;
     current_right_pane_dialog=NULL;
     pane_docked=true;
-    current_lib="";
-    floating_window=NULL;
 
     left_pane_width=150;
     stored_divider_pos=left_pane_width;
 
-    main_vb          = manage(new class Gtk::VBox());
+    navbar=NULL;
+    needs_saving_icon=NULL;
+    main_statusbar_txt=NULL;
 
-    main_menubar1    = manage(new class Gtk::MenuBar());
-    hpaned           = manage(new class Gtk::HPaned());
-    main_statusbar   = manage(new class Gtk::Statusbar());
+    main_vb          = new class Gtk::VBox();
+    main_menubar1    = new class Gtk::MenuBar();
+    save_statusbar   = new class Gtk::Viewport();
+    main_statusbar   = new class Gtk::Viewport();
    
+    hpaned           = new class Gtk::HPaned();
 
     hpaned->set_handle_size(10);
     hpaned->set_gutter_size(6);
@@ -94,9 +105,21 @@ main_window::main_window() :  Gtk::Window(GTK_WINDOW_TOPLEVEL)
 
     main_vb->pack_start(*main_menubar1 , false, false );
     main_vb->pack_start(*hpaned        , true , true  );
-    main_vb->pack_start(*main_statusbar, false, false );
-    
 
+    statusbar_hbox = new class Gtk::HBox();
+    statusbar_hbox->set_spacing(6);
+
+    main_vb->pack_start(*statusbar_hbox, false, false );
+
+    statusbar_hbox->pack_start(*save_statusbar, false, false );
+    statusbar_hbox->pack_start(*main_statusbar, true,  true  );
+
+    save_statusbar->set_shadow_type(GTK_SHADOW_IN);
+    main_statusbar->set_shadow_type(GTK_SHADOW_IN);
+
+    save_statusbar->set_usize(32 ,-1);
+
+    showStatusbarMsg(" ");
 
     int sw=gdk_screen_width();
     int sh=gdk_screen_height();
@@ -113,12 +136,13 @@ main_window::main_window() :  Gtk::Window(GTK_WINDOW_TOPLEVEL)
     add(*main_vb);
 
     hpaned->show();
+    save_statusbar->show();
     main_statusbar->show();
     main_vb->show_all();
 
 
 
-    obook=new FWObjectBook();
+    obook=manage(new FWObjectBook(this));
 
 /*
  *  do not manage() left_pane_port and right_pane_port because we will
@@ -127,8 +151,10 @@ main_window::main_window() :  Gtk::Window(GTK_WINDOW_TOPLEVEL)
     left_pane_port     = new Gtk::ScrolledWindow();
     Gtk::Viewport *vp  = manage( new Gtk::Viewport() );
     
-    ((Gtk::ScrolledWindow*)left_pane_port)->set_policy(GTK_POLICY_AUTOMATIC, 
-						       GTK_POLICY_AUTOMATIC);
+//    ((Gtk::ScrolledWindow*)left_pane_port)->set_policy(GTK_POLICY_AUTOMATIC, 
+//						       GTK_POLICY_AUTOMATIC);
+    ((Gtk::ScrolledWindow*)left_pane_port)->set_policy(GTK_POLICY_NEVER, 
+						       GTK_POLICY_NEVER);
     ((Gtk::ScrolledWindow*)left_pane_port)->add(*vp);
     vp->add( *obook );
 
@@ -142,12 +168,16 @@ main_window::main_window() :  Gtk::Window(GTK_WINDOW_TOPLEVEL)
 			  &main_window::on_left_pane_size_allocate_event));
     hpaned->pack1( *left_pane_port , false, false );
 
+    right_vb  = new class Gtk::VBox();
+
+    hpaned->pack2( *right_vb , true, true);
+
     right_pane_port = new Gtk::Viewport();
     right_pane_port->add( *(new Gtk::Label("")) ); // placeholder
     right_pane_port->size_allocate.connect(SigC::slot(this, 
 			  &main_window::on_right_pane_size_allocate_event));
-    hpaned->pack2( *right_pane_port , true, true);
 
+    right_vb->pack_start(*right_pane_port, true, true);
 
     main_menubar1->set_shadow_type( GTK_SHADOW_NONE );  // glade bug?
 
@@ -162,13 +192,117 @@ main_window::main_window() :  Gtk::Window(GTK_WINDOW_TOPLEVEL)
 
     build_menu();  // needs AccelGroup object 
 
-
     delete_event.connect( SigC::slot(this,&main_window::on_delete_event));
+
+    if ( ! Preferences::global_prefs->getOptBool("/FWBuilderPreferences/UI/HideNavigationBar") )
+        showNavbar();
 }
 
-void main_window::setCurrentLibrary(const string &lib) 
-{ 
-    current_lib=lib; 
+main_window::~main_window()
+{
+    if ( Preferences::global_prefs->getOptBool("/FWBuilderPreferences/UI/RememberWindowPositionAndSize") ) 
+	saveWindowConfiguration();
+
+    try
+    {
+	Preferences::global_prefs->savePrefs();
+    } catch(FWException &ex)
+    {
+	MessageDialog::Error(_("Error saving preferences."),ex.toString(),this);
+    }
+
+    left_pane_port->hide();
+    delete left_pane_port;
+
+    right_pane_port->hide();
+    delete right_pane_port;
+
+    if (navbar!=NULL) delete navbar;
+
+    delete right_vb;
+
+    delete save_statusbar;
+    delete main_statusbar;
+    delete statusbar_hbox;
+    delete hpaned;
+    delete main_menubar1;
+}
+
+/*
+ * This event is generated in responce to the window manager's signal
+ * sent to the window when user clicks "close" button on window's
+ * title bar 
+ */
+gint main_window::on_delete_event(GdkEventAny* ev)
+{
+    if (windows.size()>1) {
+        if ( safe_to_close_right_pane()) {
+            windows.remove(this);
+            return false;
+        }
+        return true;
+    }
+
+// this is the last window
+    if ( checkAndSaveData() ) {
+        windows.remove(this);
+        Gtk::Main::quit();
+	return false;
+    }
+    return true;
+}
+
+void main_window::on_close_activate()
+{   
+    if (windows.size()>1) {
+        if ( safe_to_close_right_pane()) {
+            windows.remove(this);
+            delete this;
+        }
+        return;
+    }
+
+// this is the last window
+    if ( checkAndSaveData() ) {
+        windows.remove(this);
+        Gtk::Main::quit();
+    }
+}
+
+void main_window::on_exit1_activate()
+{   
+    if ( checkAndSaveData() ) {
+//        ClearRightPane(false);
+	Gtk::Main::quit();
+    }
+}
+
+
+void main_window::showNavbar()
+{
+    if (navbar==NULL) {
+        navbar=new Navbar(this);
+        right_vb->pack_start(*navbar, false, false );
+        right_vb->reorder_child(*navbar,0);
+    }
+}
+
+void main_window::hideNavbar()
+{
+    if (navbar!=NULL) {
+        right_vb->remove(*navbar);
+        navbar=NULL;
+    }
+}
+
+void main_window::updateNavbar(libfwbuilder::FWObject* obj)
+{
+    if (navbar!=NULL) navbar->update(obj);
+}
+
+void main_window::updateNavbar(const std::string &id)
+{
+    if (navbar!=NULL) navbar->update(id);
 }
 
 void main_window::on_left_pane_size_allocate_event(GtkAllocation *all)
@@ -183,22 +317,55 @@ void main_window::on_right_pane_size_allocate_event(GtkAllocation *all)
     right_pane_height = all->height;
 }
 
-void main_window::LoadObjDB(const string &data_file)
+void main_window::LoadObjDB(const string &data_file,bool check_perm)
 {   
-    g_assert(FWObjectDatabaseGUI::getInstance() != NULL);
+    assert(FWObjectDatabaseGUI::getInstance() != NULL);
 
     try
     {
-        FWObjectDatabaseGUI::getInstance()->load(data_file);
+        FWObjectDatabaseGUI::getInstance()->load(data_file,check_perm);
     } catch(FWException &ex)
     {
-	MessageDialog::LongTextError("Error loading data file. ",
-				     ex.toString());
+        string trans = ex.getProperties()["failed_transformation"];
+        string elem  = ex.getProperties()["failed_element"];
+
+        if(!trans.empty() && !elem.empty())
+        {
+            string msg = _("Exception: : ") + ex.toString();
+            if(!trans.empty())
+                msg+=_("\nFailed transformation : ") + trans;
+            if(!elem.empty())
+                msg+=_("\nXML element : ") + elem;
+
+            MessageDialog::LongTextError(_("Error loading data file. "), msg, this);
+        } else
+            MessageDialog::Error(_("Error loading data file. "), ex.toString(), this);
     }
-	
-    obook->clearSelection();
-    obook->build();
-    setCurrentLibrary("");
+
+    showSaveStatusbar(false);
+    refreshAllObjectBooks();
+
+//    setCurrentLibrary("");
+}
+
+main_window *main_window::createNewMainWindow()
+{
+    main_window* mw = new main_window();
+    if ( Preferences::global_prefs->getOptBool(
+	     "/FWBuilderPreferences/UI/RememberWindowPositionAndSize") ) 
+    {
+	mw->restoreWindowConfiguration();
+    } 
+    mw->restoreWindowPosition();
+    mw->obook->build();
+    mw->show_all();
+    return mw;
+}
+
+void main_window::on_new_wnd_activate()
+{
+    main_window* mw=createNewMainWindow();
+    mw->schedule_open_object(FWObjectDatabaseGUI::db->getRootId(),"");
 }
 
 void main_window::on_new1_activate()
@@ -207,17 +374,53 @@ void main_window::on_new1_activate()
     {
         string dbTemplateFile = TEMPLATE_DIR  "/" + FWObjectDatabaseGUI::DB_TEMPLATE_FILE_NAME;
 	ClearRightPane(true);
-	LoadObjDB(dbTemplateFile);
+	LoadObjDB(dbTemplateFile,false);
         FWObjectDatabase::db->setFileName("");
+        setTitle("");
+    }
+}
+
+void main_window::openFile(const string &file,bool update_recent)
+{
+    if (safe_to_close_right_pane()) 
+    {
+	ClearRightPane(true);
+	LoadObjDB(file);
+        setTitle( FWObjectDatabaseGUI::getInstance()->getFileName() );
+        if (update_recent) updateRecentlyOpenedFiles( FWObjectDatabaseGUI::getInstance()->getFileName() );
     }
 }
 
 void main_window::on_open1_activate()
 {   
-    if (safe_to_close_right_pane()) {
-	ClearRightPane(true);
-	LoadObjDB("");
-    }
+    FileSel *fs=new FileSel(_("Select file"),
+                            Preferences::global_prefs->getWdir(),
+                            "*.xml");
+    string df=fs->run();
+    delete fs;
+    if (df=="") return;
+
+    openFile(df,true);
+}
+
+void main_window::on_open_recent_file1_activate()
+{
+    openFile( Preferences::global_prefs->getOptStr("/FWBuilderPreferences/History/File1") ,false );
+}
+
+void main_window::on_open_recent_file2_activate()
+{
+    openFile( Preferences::global_prefs->getOptStr("/FWBuilderPreferences/History/File2") ,false );
+}
+
+void main_window::on_open_recent_file3_activate()
+{
+    openFile( Preferences::global_prefs->getOptStr("/FWBuilderPreferences/History/File3") ,false );
+}
+
+void main_window::on_open_recent_file4_activate()
+{
+    openFile( Preferences::global_prefs->getOptStr("/FWBuilderPreferences/History/File4") ,false );
 }
 
 void main_window::on_save1_activate()
@@ -227,9 +430,11 @@ void main_window::on_save1_activate()
     try
     {
         FWObjectDatabaseGUI::getInstance()->save(); 
+        showStatusbarMsg(string(_("Saved as: "))+FWObjectDatabaseGUI::getInstance()->getFileName() );
+        showSaveStatusbar(false);
     } catch(FWException &ex)
     {
-	MessageDialog::Error("Error saving data file.",ex.toString());
+	MessageDialog::Error(_("Error saving data file."),ex.toString(),this);
     }
 }
 
@@ -240,45 +445,32 @@ void main_window::on_saveas1_activate()
     try
     {
         FWObjectDatabaseGUI::getInstance()->saveAs(); 
+        setTitle( FWObjectDatabaseGUI::getInstance()->getFileName() );
+        showStatusbarMsg(string(_("Saved as: "))+FWObjectDatabaseGUI::getInstance()->getFileName() );
+        showSaveStatusbar(false);
     } catch(FWException &ex)
     {
-	MessageDialog::Error("Error saving data file.",ex.toString());
+        if ( ex.toString()!="" )
+            MessageDialog::Error(_("Error saving data file."),ex.toString(),this);
     }
 }
 
 bool main_window::checkAndSaveData()
 {
     if ( ! safe_to_close_right_pane()) return false;
-    if (FWObjectDatabase::db->isDirty(true) ) {
-        try  {
+    if (FWObjectDatabase::db->isDirty(true) ) 
+    {
+        try  
+        {
             if (! FWObjectDatabaseGUI::getInstance()->saveIfModified() ) return false;  // "Cancel"
         } catch(FWException &ex)  {
-	    MessageDialog::Error("Error saving data file.",ex.toString());
+	    MessageDialog::Error(_("Error saving data file."),ex.toString(),this);
+            return false;
         }
     }
     return true;
 }
 
-/*
- * This event is generated in responce to the window manager's signal
- * sent to the window when user clicks "close" button on window's
- * title bar 
- */
-gint main_window::on_delete_event(GdkEventAny* ev)
-{
-    if ( checkAndSaveData() ) {
-	Gtk::Main::quit();
-	return false;
-    }
-    return true;
-}
-
-void main_window::on_exit1_activate()
-{   
-    if ( checkAndSaveData() )
-	Gtk::Main::quit();
-}
-
 void main_window::on_print_activate()
 {   
     if(checkAndSaveData())
@@ -289,10 +481,11 @@ void main_window::on_print_activate()
         );
         try
         {
+            pd->set_transient_for( *this );
             pd->run();
         } catch(FWException &ex)
         {
-            MessageDialog::Error("Print error.",ex.toString());
+            MessageDialog::Error(_("Print error."),ex.toString(),this);
         }
 	delete pd;
     }
@@ -306,25 +499,57 @@ void main_window::on_options_activate()
 {   
     bool show_properties=Preferences::global_prefs->getOptBool(
 	"/FWBuilderPreferences/UI/ShowObjectPropertiesInTree");
-    bool hide_navbar=Preferences::global_prefs->getOptBool(
-	"/FWBuilderPreferences/UI/HideNavigationBar");
-
+    bool show_icons=Preferences::global_prefs->getOptBool(
+	"/FWBuilderPreferences/UI/ShowIconsInTree");
 
     OptionsDlg *pd=new OptionsDlg();
-    pd->run();
+    bool ok=pd->run();
     delete pd;
 
-    obook->setMode(
-	Preferences::global_prefs->getOptStr(
-	    "/FWBuilderPreferences/UI/ObjectTreeMode") );
+    if (ok) 
+    {
+	obook->setMode(Preferences::global_prefs->getOptStr(
+			   "/FWBuilderPreferences/UI/ObjectTreeMode") );
 
-    if (show_properties!=Preferences::global_prefs->getOptBool(
-	    "/FWBuilderPreferences/UI/ShowObjectPropertiesInTree") )
-	obook->build();
+	if (show_properties!=Preferences::global_prefs->getOptBool(
+		"/FWBuilderPreferences/UI/ShowObjectPropertiesInTree") 
+            ||
+            show_icons!=Preferences::global_prefs->getOptBool(
+                "/FWBuilderPreferences/UI/ShowIconsInTree") 
+        )
+	    obook->build();
 
-    if (hide_navbar!=Preferences::global_prefs->getOptBool(
-	    "/FWBuilderPreferences/UI/HideNavigationBar") )
-	main_w->schedule_open_object(main_w->nowShowingObjectId());
+        if (Preferences::global_prefs->getOptBool(
+                "/FWBuilderPreferences/UI/HideNavigationBar") )
+            hideNavbar();
+        else
+            showNavbar();
+
+        Gtk::MenuBar *main_menubar=getMainMenuBar();
+        Gtk::Widget *itm;
+
+        itm=find_widget("sort_by_prop",main_menubar);
+        if (itm) itm->set_sensitive(
+            (Preferences::global_prefs->getOptBool(
+                "/FWBuilderPreferences/UI/ShowObjectPropertiesInTree")));
+
+/*
+ * can't simply call schedule_open_object because if currently shown dialog
+ * has unsaved data, it will be lost
+ */
+	BuiltinDialog *dlg=getRightPaneDialog();
+	if (dlg) 
+        {
+/*
+ * User might have changes something in certificates, let's refresh list if
+ * what we have in the right pane is firewall dialog
+ */
+	    DialogPlugin*  dp=dlg->getDialog();
+	    if (dp!=NULL && dynamic_cast<FirewallDialog*>(dp)!=NULL) {
+		dynamic_cast<FirewallDialog*>(dp)->fillListOfCertificates();
+	    }
+	}
+    }
 }
 
 void main_window::on_about1_activate()
@@ -339,29 +564,68 @@ void main_window::on_release_notes_activate()
     string file=DOCDIR "/ReleaseNotes_" VERSION;
     std::ifstream rn( file.c_str() );
     if (!rn) {
-	MessageDialog::Error("Can not open Release Notes file " + file );
+	MessageDialog::Error(_("Can not open Release Notes file ") + file , this);
 	return;
     }
     string buf, s;
 
     while (getline(rn,s)) { buf+=s; buf+="\n"; }
 
-    MessageDialog::LongTextInfo( buf );
+    MessageDialog::LongTextInfo( buf , this);
 }
 
+void  main_window::setTitle(const string &msg)
+{
+    for (list<main_window*>::iterator i=windows.begin(); i!=windows.end(); ++i) 
+    {
+        (*i)->set_title(_("Firewall Builder: ")+msg);
+    }
+}
+
+void  main_window::showSaveStatusbar(bool needs_saving)
+{
+    if (FWObjectDatabase::db->isDirty(true) ) 
+    {
+        for (list<main_window*>::iterator i=windows.begin(); i!=windows.end(); ++i) {
+            if ( (*i)->needs_saving_icon ==NULL ) {
+                string icn = Resources::global_res->getIconPath("Floppy");
+                SimplePixmap *pix=new SimplePixmap(icn);
+                (*i)->save_statusbar->remove();
+                (*i)->save_statusbar->add(*pix);
+                pix->show();
+            }
+        }
+    }
+    else 
+    {
+        for (list<main_window*>::iterator i=windows.begin(); i!=windows.end(); ++i) {
+            (*i)->save_statusbar->remove();
+        }
+    }
+}
+
+
 void main_window::showStatusbarMsg(const string& msg)
 {
-  main_statusbar->push(1,msg);
+    clearStatusbar();
+    for (list<main_window*>::iterator i=windows.begin(); i!=windows.end(); ++i) 
+    {
+        (*i)->main_statusbar->remove();
+        (*i)->main_statusbar->add_label(string("  ")+msg );
+    }
 }
 
 void main_window::clearStatusbar()
 {
-  main_statusbar->pop(1);
+    for (list<main_window*>::iterator i=windows.begin(); i!=windows.end(); ++i) 
+    {
+        (*i)->main_statusbar->remove();
+    }
 }
 
-PolicyList* main_window::extractPolicyList()
+RuleSetList* main_window::getRuleSetListWidget()
 {
-    PolicyList *pl=NULL;
+    RuleSetList *pl=NULL;
 
     if (current_right_pane_dialog) {
 	DialogPlugin *dp= current_right_pane_dialog->getDialog();
@@ -372,8 +636,8 @@ PolicyList* main_window::extractPolicyList()
 	if (dp->get_name()=="nat_dialog")
 	    pl=((NATDialog*)dp)->getPolicyList();
 
-	if (dp->get_name()=="firewall_dialog")
-	    pl=((FirewallDialog*)dp)->getPolicyList();
+//	if (dp->get_name()=="firewall_dialog")
+//	    pl=((FirewallDialog*)dp)->getPolicyList();
     }
     return pl;
 }
@@ -384,42 +648,42 @@ PolicyList* main_window::extractPolicyList()
 void main_window::on_insert_top()    
 { 
     if (current_right_pane_dialog==NULL) return;
-    PolicyList *pl=extractPolicyList();
+    RuleSetList *pl=getRuleSetListWidget();
     if (pl!=NULL) pl->insertRuleAtTop(); 
 }
 
 void main_window::on_append_bottom() 
 { 
     if (current_right_pane_dialog==NULL) return;
-    PolicyList *pl=extractPolicyList();
+    RuleSetList *pl=getRuleSetListWidget();
     if (pl!=NULL) pl->appendRuleAtBottom();  
 }
 
 void main_window::on_add_rule()
 {
     if (current_right_pane_dialog==NULL) return;
-    PolicyList *pl=extractPolicyList();
+    RuleSetList *pl=getRuleSetListWidget();
     if (pl!=NULL) pl->appendRuleAfter(-1);
 }
 
 void main_window::on_remove_rule()   
 { 
     if (current_right_pane_dialog==NULL) return;
-    PolicyList *pl=extractPolicyList();
+    RuleSetList *pl=getRuleSetListWidget();
     if (pl!=NULL) pl->delRule(-1);
 }
 
 void main_window::on_move_up()       
 { 
     if (current_right_pane_dialog==NULL) return;
-    PolicyList *pl=extractPolicyList();
+    RuleSetList *pl=getRuleSetListWidget();
     if (pl!=NULL) pl->moveRuleUp(-1);
 }
 
 void main_window::on_move_down()     
 {
     if (current_right_pane_dialog==NULL) return;
-    PolicyList *pl=extractPolicyList();
+    RuleSetList *pl=getRuleSetListWidget();
     if (pl!=NULL) pl->moveRuleDown(-1);
 }
 
@@ -436,9 +700,15 @@ void main_window::on_build_autorules()
 //		ClearRightPane(false);
 
 	    StandardRulesDruid *dr=new StandardRulesDruid( obj->getId(),
-						   "Standard policy rules",
+						   _("Standard policy rules"),
 		       Resources::global_res->getIconPath("RulesDruidLogo") );
+
+
 	    dr->show_all();
+#ifdef __MINGW32__
+            __fwb_win32_loadIcon("FWBuilderWin32Ico",dr);
+#endif
+
 	}
 
     }
@@ -448,31 +718,52 @@ void main_window::on_compile()
 {
     if (current_right_pane_dialog==NULL) return;
 
-    if (safe_to_close_right_pane()) {
+    if (safe_to_close_right_pane()) 
+    {
 	FWObject   *obj=nowShowingObject();
-	if (obj) {
-	    if (Policy::isA(obj)) obj=obj->getParent();
-	    if (NAT::isA(obj))    obj=obj->getParent();
-
-	    if (Firewall::isA(obj))   {
+	if (obj) 
+        {
+            while (obj!=NULL && !Firewall::isA(obj)) obj=obj->getParent();
+            
+	    if (Firewall::isA(obj))   
+            {
 		on_save1_activate();
 // call compiler now
 
 		Firewall  *fw=Firewall::cast(obj);
 		FWOptions *fwopt=fw->getOptionsObject();
-		
+
+/*
+ * I want to be able to specify custom compiler for firewall with
+ * no platform (e.g. for experiments)
+ */
 		string compiler=fwopt->getStr("compiler");
-		if (compiler=="") {
-		    compiler=Resources::global_res->getCompiler(
-			obj->getStr("platform"));
+		if (compiler=="") 
+                {
+                    compiler=Resources::platform_res[fw->getStr("platform")]->getCompiler();
 		}
-            
+
+		if (compiler=="")
+                {
+                    MessageDialog::Error(
+_("Firewall platform is not specified in this object.\n\
+Can't compile firewall policy.")
+                    );
+                    return;
+                }
+                string wdir=Preferences::global_prefs->getWdir();
+                if (wdir.empty())
+                {
+                    string file=FWObjectDatabase::db->getFileName();
+                    string::size_type n=file.rfind("/");
+                    if (n!=string::npos) wdir=file.substr(0,n);
+                }
 		ExecDialog *cmpl=
 		    new ExecDialog(compiler , 
 				   fwopt->getStr("cmdline") ,
 				   FWObjectDatabase::db->getFileName(),
-				   obj->getName() ,
-				   Preferences::global_prefs->getWdir()
+				   fw->getName() ,
+				   wdir
 		    );
 
 		cmpl->execute();
@@ -486,31 +777,49 @@ void main_window::on_install()
 {
     if (current_right_pane_dialog==NULL) return;
 
-    if (safe_to_close_right_pane()) {
+    if (safe_to_close_right_pane()) 
+    {
 	FWObject   *obj=nowShowingObject();
-	if (obj) {
-	    if (Policy::isA(obj)) obj=obj->getParent();
-	    if (NAT::isA(obj))    obj=obj->getParent();
+	if (obj) 
+        {
+            while (obj!=NULL && !Firewall::isA(obj) ) obj=obj->getParent();
 
-	    if (Firewall::isA(obj))   {
+	    if (obj!=NULL && Firewall::isA(obj))   
+            {
 		on_save1_activate();
 // call install script now
 
-		Firewall  *fw           = Firewall::cast(obj);
-		FWOptions *fwopt = fw->getOptionsObject();
-		
-		string inst_script=fwopt->getStr("inst_script");
-		if (inst_script=="") {
-// TODO: may be Error dialog "install script is not defined"
-		    return ;
+		Management *mgmt=(Firewall::cast(obj))->getManagementObject();
+		assert(mgmt!=NULL);
+		PolicyInstallScript *pis   = mgmt->getPolicyInstallScript();
+
+		if ( ! pis->isEnabled() )
+                {
+		    MessageDialog::Error(_("Install script is disabled"),this);
+		    return;
 		}
+
+		if ( pis->getCommand()=="" )
+                {
+		    MessageDialog::Error(_("Install script is not configured"),this);
+		    return;
+                }
+
+		string inst_script=pis->getCommand();
             
+                string wdir=Preferences::global_prefs->getWdir();
+                if (wdir.empty())
+                {
+                    string file=FWObjectDatabase::db->getFileName();
+                    string::size_type n=file.rfind("/");
+                    if (n!=string::npos) wdir=file.substr(0,n);
+                }
 		ExecDialog *inst=
 		    new ExecDialog(inst_script , 
-				   fwopt->getStr("inst_cmdline") ,
+				   pis->getArguments(), 
 				   FWObjectDatabase::db->getFileName(),
 				   obj->getName() ,
-				   Preferences::global_prefs->getWdir()
+				   wdir
 		    );
 
 		inst->execute();
@@ -522,8 +831,93 @@ void main_window::on_install()
 
 void main_window::RefreshRightPane()
 {
-    PolicyList    *pl=extractPolicyList();
-    if (pl) pl->schedule_rebuild(0);
+    BuiltinDialog *bd=getRightPaneDialog();
+    if (bd!=NULL) bd->getDialog()->conditionalRefresh();
+}
+
+void  main_window::refreshAllWindows()
+{
+    for (list<main_window*>::iterator i=windows.begin(); i!=windows.end(); ++i) {
+        BuiltinDialog *bd=(*i)->getRightPaneDialog();
+        if (bd!=NULL) {
+            bd->getDialog()->conditionalRefresh();
+        }
+    }
+}
+
+void main_window::conditionalRefreshAllDialogs(libfwbuilder::FWObject* obj,
+                                               main_window *mwin)
+{
+    for (list<main_window*>::iterator i=windows.begin(); i!=windows.end(); ++i) 
+    {
+        if (mwin!=NULL && (*i)==mwin) continue;
+        BuiltinDialog *bd=(*i)->getRightPaneDialog();
+        if (bd!=NULL) {
+            bd->getDialog()->conditionalRefresh(obj);
+        }
+    }
+}
+
+
+void  main_window::refreshAllObjectBooks()
+{
+    for (list<main_window*>::iterator i=windows.begin(); i!=windows.end(); ++i) 
+    {
+        (*i)->getObjectBook()->rebuild();
+    }
+}
+
+void  main_window::sortSubtreeInAllObjectBooks(const string &id,int col)
+{
+    for (list<main_window*>::iterator i=windows.begin(); i!=windows.end(); ++i) 
+    {
+        (*i)->getObjectBook()->sortSubtree(id,col);
+    }
+}
+
+
+void  main_window::insertInAllObjectBooks(FWObject *o)
+{
+    for (list<main_window*>::iterator i=windows.begin(); i!=windows.end(); ++i) 
+    {
+	(*i)->getObjectBook()->showPage(0);  // User-defined objects
+        (*i)->getObjectBook()->insertObject(o);
+    }
+}
+
+void  main_window::removeFromAllObjectBooks(FWObject *o)
+{
+    for (list<main_window*>::iterator i=windows.begin(); i!=windows.end(); ++i) 
+    {
+        (*i)->getObjectBook()->removeObject(o);
+    }
+}
+     
+void  main_window::changeLabelInAllObjectBooks(const std::string &id)
+{
+    for (list<main_window*>::iterator i=windows.begin(); i!=windows.end(); ++i) 
+    {
+        (*i)->getObjectBook()->changeTreeLabel( id );
+    }
+}
+
+main_window* main_window::getMainWindowForWidget(Gtk::Widget *w)
+{
+    Gtk::Widget *p=w;
+    while (p!=NULL && dynamic_cast<main_window*>(p)==NULL) p=p->get_parent();
+    return dynamic_cast<main_window*>(p);
+}
+
+main_window* main_window::getAnotherMainWindowForWidget(Gtk::Widget *w)
+{
+    main_window *mw=getMainWindowForWidget(w);
+    for (list<main_window*>::iterator i=windows.begin(); i!=windows.end(); ++i) {
+        if (mw!=(*i)) return (*i);
+    }
+/*
+ * there is only one window and widget w is in it, create new window.
+ */
+    return createNewMainWindow();
 }
 
 string main_window::nowShowingObjectId()
@@ -547,9 +941,17 @@ void main_window::on_tools_scan()
 	Resources::global_res->getResourceStr("/FWBuilderResources/Paths/Icndir")+"/"+
 	Resources::global_res->getResourceStr("/FWBuilderResources/UI/Icons/Discover");
 
-    DiscoveryDruid *dr=new DiscoveryDruid("Objects Discovery",icn);
+
+
+    DiscoveryDruid *dr=new DiscoveryDruid(this,_("Objects Discovery"),icn);
+
+
 
     dr->show_all();
+#ifdef __MINGW32__
+    __fwb_win32_loadIcon("FWBuilderWin32Ico",dr);
+#endif
+
 }
 
 /*
@@ -626,6 +1028,9 @@ void main_window::writeGeometry(windowGeometry &c)
     Preferences::global_prefs->setOptInt(
 	"/FWBuilderPreferences/UI/TreeWindowH",c.trwnd->height);
     
+
+
+    
     Preferences::global_prefs->setOptInt(
 	"/FWBuilderPreferences/UI/ObjWindowX" ,c.objwnd->x);
     Preferences::global_prefs->setOptInt(
@@ -641,22 +1046,10 @@ void main_window::restoreWindowConfiguration()
 {
     windowGeometry g;
 
-//    gdk_window_get_geometry(NULL, &i, &i, &w_display, &h_display, &i);
-
     readGeometry(g);
 
-    bool float_pane=Preferences::global_prefs->getOptBool(
-	"/FWBuilderPreferences/UI/FloatObjectTree");
-
-    if (float_pane==pane_docked)  on_float_pane();
-
-    if (pane_docked) {
-	setWindowConfiguration(g.wnd);
-	setDividerPosition(g.divider);
-    } else {
-	setTreeWndConfiguration(g.trwnd);
-	setObjWndConfiguration( g.objwnd);
-    }
+    setWindowConfiguration(g.wnd);
+    setDividerPosition(g.divider);
 }
 
 void main_window::restoreWindowPosition()
@@ -670,18 +1063,9 @@ void main_window::restoreWindowPosition()
     gdk_window_get_geometry(NULL, &i, &i, &w_display, &h_display, &i);
 
     if (g.wnd->x>0      && g.wnd->x<w_display-100 && 
-	g.wnd->y>0      && g.wnd->y<h_display-100 &&
-	g.trwnd->x>0    && g.trwnd->x<w_display-100 &&
-	g.trwnd->y>0    && g.trwnd->y<h_display-100 &&
-	g.objwnd->x>0   && g.objwnd->x<w_display-100 &&
-	g.objwnd->y>0   && g.objwnd->y<h_display-100 ) {
-	    
-	if (pane_docked) {
-	    setWindowPosition(g.wnd->x,g.wnd->y);
-	} else {
-	    setTreeWndPosition(g.trwnd->x  , g.trwnd->y );
-	    setObjWndPosition( g.objwnd->x , g.objwnd->y);
-	}
+	g.wnd->y>0      && g.wnd->y<h_display-100 )
+    {
+        setWindowPosition(g.wnd->x,g.wnd->y);
     }
 }
 
@@ -696,157 +1080,17 @@ void main_window::saveWindowConfiguration()
  *  In this case its size and coordinates are going to be equal to 0 and we
  *  should not store them in preferences file
  */
-    if (is_visible()) {
+    if (is_visible()) 
+    {
 	getWindowConfiguration(g.wnd);
-	getTreeWndConfiguration(g.trwnd);
-	getObjWndConfiguration( g.objwnd);
 	g.divider=getDividerPosition();
-    } else {
+    } else 
+    {
 	readGeometry(g);
     }
 
     writeGeometry(g);
 
-
-    Preferences::global_prefs->setOptBool(
-	"/FWBuilderPreferences/UI/FloatObjectTree" ,
-	! pane_docked );
-	
-    try
-    {
-	Preferences::global_prefs->savePrefs();
-    } catch(FWException &ex)
-    {
-	MessageDialog::Error("Error saving preferences.",ex.toString());
-    }
-
-}
-
-void main_window::on_float_pane()
-{
-    int div;
-    Gdk_Rectangle wndr, treer, objr;
-
-    getWindowConfiguration(wndr);
-    getTreeWndConfiguration(treer);
-    getObjWndConfiguration( objr);
-    div=getDividerPosition();
-
-    if (pane_docked) {
-	floating_window=manage(new Gtk::Window());
-	floating_window->set_name("FloatingPane");
-        floating_window->set_policy(false,true,false);
-
-
-	right_pane_port->reparent( *floating_window );
-
-	stored_divider_pos=treer->width;
-
-	left_pane_port->hide();
-
-	hpaned->remove( *left_pane_port );
-	main_vb->remove( *hpaned );
-
-	main_vb->pack_start(*left_pane_port        , true , true  );
-	main_vb->reorder_child( *left_pane_port , 1 );
-
-//	main_tbl->attach(*left_pane_port , 0,1,1,2 , 
-//			 GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND);
-
-	left_pane_port->show();
-
-	Gtk::MenuItem *itm=(Gtk::MenuItem*)( find_widget("float_pane",
-							 main_menubar1) );
-	assert (itm!=NULL);
-	Gtk::Label *lbl=(Gtk::Label*)(itm->get_child());
-	assert(lbl!=NULL);
-	lbl->set_text("Dock Object Dialog");
-
-	if (is_visible()) 
-	    floating_window->show();
-
-	floating_window->delete_event.connect( SigC::slot(this,
-			      &main_window::on_floated_pane_delete_event));
-	pane_docked=false;
-
-	setObjWndConfiguration(objr);
-	setTreeWndConfiguration(treer);
-
-    } else {
-	on_dock_pane();
-    }
-}
-
-gint main_window::on_dock_pane()
-{
-    int ww,wh;
-    int sw,sh;
-
-    assert(floating_window!=NULL);
-
-    Gdk_Rectangle wndr, treer, objr;
-
-    getWindowConfiguration(wndr);
-    getTreeWndConfiguration(treer);
-    getObjWndConfiguration( objr);
-
-
-    sw=gdk_screen_width();
-    sh=gdk_screen_height();
-
-    GtkWidget *widget=GTK_WIDGET(floating_window->gtkobj());
-    if (widget->window)
-	gdk_window_get_size(widget->window, &ww, &wh);
-
-
-    main_vb->remove( *left_pane_port );
-
-    hpaned  = manage(new class Gtk::HPaned());
-
-    hpaned->set_handle_size(10);
-    hpaned->set_gutter_size(6);
-    hpaned->set_position(-1);
-
-
-    hpaned->pack1( *left_pane_port , false, false );
-
-    right_pane_port->unparent();
-    hpaned->pack2( *right_pane_port , true, true );
-
-    main_vb->pack_start(*hpaned        , true , true  );
-    main_vb->reorder_child( *hpaned , 1 );
-
-    hpaned->show();
-
-    int width;
-    if (wndr->width+ww>=sw) width=wndr->width;// combined window would be too wide
-    else                    width=wndr->width+ww;
-    
-    wndr->width=width;
-    setWindowConfiguration(wndr);
-
-    setDividerPosition( 150 );
-
-
-    Gtk::MenuItem *itm=(Gtk::MenuItem*)( find_widget("float_pane",
-							 main_menubar1) );
-    assert (itm!=NULL);
-    Gtk::Label *lbl=(Gtk::Label*)(itm->get_child());
-    assert(lbl!=NULL);
-    lbl->set_text("Float Object Dialog");
-
-    floating_window->hide();
-
-    pane_docked=true;
-    floating_window=NULL;
-
-    return false;
-}
-
-gint main_window::on_floated_pane_delete_event(GdkEventAny* ev)
-{
-    Gtk::Main::idle.connect(slot(this,&main_window::on_dock_pane));
-    return true; // deny closing window now
 }
 
 void main_window::getWindowConfiguration(Gdk_Rectangle &r)
@@ -878,64 +1122,6 @@ void main_window::setWindowPosition(gint16 x , gint16 y)
 }
 
 
-void main_window::getTreeWndConfiguration(Gdk_Rectangle &r)
-{
-    getWindowConfiguration(r);
-    if (pane_docked) {
-	r->width=left_pane_width;
-    }
-}
-
-void main_window::setTreeWndConfiguration(Gdk_Rectangle &r)
-{
-    setWindowConfiguration(r);
-}
-
-void main_window::setTreeWndPosition(gint16 x , gint16 y)
-{
-    if (is_visible()) {
-	GtkWidget *widget=GTK_WIDGET(gtkobj());
-	gdk_window_move(widget->window, x, y);
-    }
-}
-
-
-void main_window::getObjWndConfiguration(Gdk_Rectangle &r)
-{
-    if (pane_docked) {
-	getWindowConfiguration(r);
-	r->x     += left_pane_width;
-	r->width -= left_pane_width;
-    } else {
-	GtkWidget *widget=GTK_WIDGET(floating_window->gtkobj());
-	if (widget->window) {
-	    int a,b;
-	    gdk_window_get_position(widget->window, &a, &b);
-	    r->x=a; r->y=b;
-	    gdk_window_get_size(widget->window, &a, &b);
-	    r->width=a; r->height=b;
-	}
-    }
-}
-
-void main_window::setObjWndConfiguration(Gdk_Rectangle &r)
-{
-    if (! pane_docked) {
-	setObjWndPosition( r->x ,  r->y);
-	floating_window->set_default_size( r->width , r->height );
-    }
-}
-
-void main_window::setObjWndPosition(gint16 x , gint16 y)
-{
-    if (! pane_docked) {
-	if (floating_window->is_visible()) {
-	    GtkWidget *widget=GTK_WIDGET(floating_window->gtkobj());
-	    gdk_window_move(widget->window, x, y);
-	}
-    }
-}
-
 int  main_window::getDividerPosition()         { return left_pane_width;     }
 void main_window::setDividerPosition(int size) 
 { 
@@ -946,11 +1132,9 @@ void main_window::setDividerPosition(int size)
 void main_window::show_impl()
 {
     Gtk::Widget::show_impl();
-    if (floating_window!=NULL) floating_window->show();
 }
 
 void main_window::show_all_impl()
 {
     Gtk::Widget::show_all_impl();
-    if (floating_window!=NULL) floating_window->show_all();
 }
